@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -67,6 +67,9 @@ namespace Orthanc
 
     void AddChunk(const std::string& chunk);
 
+    // The source content will be emptied
+    void AddChunkDestructive(std::string& chunk);
+
     void Flatten(std::string& result);
   };
 }
