@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2018 Osimis S.A., Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -57,9 +57,16 @@ namespace Orthanc
 
     PhotometricInterpretation  photometric_;
 
+  protected:
+    explicit DicomImageInformation()
+    {
+    }
+
   public:
     explicit DicomImageInformation(const DicomMap& values);
 
+    DicomImageInformation* Clone() const;
+
     unsigned int GetWidth() const
     {
       return width_;
