@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,9 +33,11 @@
 
 #pragma once
 
+#include "../Enumerations.h"
+
 #include <stdint.h>
-#include <string>
 #include <boost/noncopyable.hpp>
+#include <json/value.h>
 
 #if !defined(ORTHANC_ENABLE_BASE64)
 #  error The macro ORTHANC_ENABLE_BASE64 must be defined
@@ -91,7 +93,7 @@ namespace Orthanc
 
     void FormatDataUriScheme(std::string& target) const
     {
-      FormatDataUriScheme(target, "application/octet-stream");
+      FormatDataUriScheme(target, MIME_BINARY);
     }
 #endif
 
@@ -108,6 +110,12 @@ namespace Orthanc
 
     bool ParseFloat(float& result) const;                                
 
-    bool ParseDouble(double& result) const;                                
+    bool ParseDouble(double& result) const;
+
+    bool ParseFirstUnsignedInteger(unsigned int& result) const;
+
+    void Serialize(Json::Value& target) const;
+
+    void Unserialize(const Json::Value& source);
   };
 }
