@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2018 Osimis S.A., Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -44,15 +44,32 @@ namespace Orthanc
   class RemoteModalityParameters
   {
   private:
-    std::string aet_;
-    std::string host_;
-    uint16_t port_;
-    ModalityManufacturer manufacturer_;
+    std::string           aet_;
+    std::string           host_;
+    uint16_t              port_;
+    ModalityManufacturer  manufacturer_;
+    bool                  allowEcho_;
+    bool                  allowStore_;
+    bool                  allowFind_;
+    bool                  allowMove_;
+    bool                  allowGet_;
+
+    void Clear();
+
+    void UnserializeArray(const Json::Value& serialized);
+
+    void UnserializeObject(const Json::Value& serialized);
 
   public:
-    RemoteModalityParameters();
+    RemoteModalityParameters()
+    {
+      Clear();
+    }
 
-    RemoteModalityParameters(const Json::Value& serialized);
+    RemoteModalityParameters(const Json::Value& serialized)
+    {
+      Unserialize(serialized);
+    }
 
     RemoteModalityParameters(const std::string& aet,
                              const std::string& host,
@@ -84,10 +101,7 @@ namespace Orthanc
       return port_;
     }
 
-    void SetPortNumber(uint16_t port)
-    {
-      port_ = port;
-    }
+    void SetPortNumber(uint16_t port);
 
     ModalityManufacturer GetManufacturer() const
     {
@@ -104,10 +118,16 @@ namespace Orthanc
       manufacturer_ = StringToModalityManufacturer(manufacturer);
     }
 
-    void FromJson(const Json::Value& modality);
+    bool IsRequestAllowed(DicomRequestType type) const;
 
-    void ToJson(Json::Value& value) const;
+    void SetRequestAllowed(DicomRequestType type,
+                           bool allowed);
 
-    void Serialize(Json::Value& target) const;
+    void Unserialize(const Json::Value& modality);
+
+    bool IsAdvancedFormatNeeded() const;
+
+    void Serialize(Json::Value& target,
+                   bool forceAdvancedFormat) const;
   };
 }
