@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -93,6 +93,7 @@
 #if ORTHANC_ENABLE_JPEG == 1
 #  include "../../Images/JpegWriter.h"
 #endif
+#include "../../Images/PamWriter.h"
 
 #include <boost/lexical_cast.hpp>
 
@@ -356,6 +357,8 @@ namespace Orthanc
   static void CopyPixels(ImageAccessor& target,
                          const DicomIntegerPixelAccessor& source)
   {
+    // WARNING - "::min()" should be replaced by "::lowest()" if
+    // dealing with float or double (which is not the case so far)
     const PixelType minValue = std::numeric_limits<PixelType>::min();
     const PixelType maxValue = std::numeric_limits<PixelType>::max();
 
@@ -632,8 +635,8 @@ namespace Orthanc
                              &dataset, frame, startFragment, &uncompressed[0],
                              uncompressed.size(), decompressedColorModel).good())
       {
-        LOG(ERROR) << "Cannot decode a palette image";
-        throw OrthancException(ErrorCode_BadFileFormat);
+        throw OrthancException(ErrorCode_BadFileFormat,
+                               "Cannot decode a palette image");
       }
 
       return DecodeLookupTable(target, info, dataset,
@@ -647,8 +650,8 @@ namespace Orthanc
                              &dataset, frame, startFragment, target->GetBuffer(), 
                              target->GetSize(), decompressedColorModel).good())
       {
-        LOG(ERROR) << "Cannot decode a non-palette image";
-        throw OrthancException(ErrorCode_BadFileFormat);
+        throw OrthancException(ErrorCode_BadFileFormat,
+                               "Cannot decode a non-palette image");
       }
 
       return target.release();
@@ -805,8 +808,8 @@ namespace Orthanc
       }
     }
 
-    LOG(ERROR) << "Cannot decode a DICOM image with the built-in decoder";
-    throw OrthancException(ErrorCode_BadFileFormat);
+    throw OrthancException(ErrorCode_BadFileFormat,
+                           "Cannot decode a DICOM image with the built-in decoder");
   }
 
 
@@ -881,7 +884,8 @@ namespace Orthanc
         else
         {
           ImageProcessing::ShiftScale(*image, static_cast<float>(-a),
-                                      255.0f / static_cast<float>(b - a));
+                                      255.0f / static_cast<float>(b - a),
+                                      true /* TODO - Consider using "false" to speed up */);
         }
 
         // If the source image is not grayscale 8bpp, convert it
@@ -951,6 +955,17 @@ namespace Orthanc
   }
 
 
+  void DicomImageDecoder::ExtractPamImage(std::string& result,
+                                          std::auto_ptr<ImageAccessor>& image,
+                                          ImageExtractionMode mode,
+                                          bool invert)
+  {
+    ApplyExtractionMode(image, mode, invert);
+
+    PamWriter writer;
+    writer.WriteToMemory(result, *image);
+  }
+
 #if ORTHANC_ENABLE_PNG == 1
   void DicomImageDecoder::ExtractPngImage(std::string& result,
                                           std::auto_ptr<ImageAccessor>& image,
