@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,6 +36,7 @@
 
 #include "../Logging.h"
 #include "../OrthancException.h"
+#include "../SystemToolbox.h"
 #include "HttpOutput.h"
 
 #include <stdio.h>
@@ -61,7 +62,7 @@ namespace Orthanc
     const UriComponents& uri,
     const Arguments& headers,
     const GetArguments& arguments,
-    const char* /*bodyData*/,
+    const void* /*bodyData*/,
     size_t /*bodySize*/)
   {
     if (!Toolbox::IsChildUri(baseUri_, uri))
@@ -77,14 +78,14 @@ namespace Orthanc
     }
 
     std::string resourcePath = Toolbox::FlattenUri(uri, baseUri_.size());
-    std::string contentType = Toolbox::AutodetectMimeType(resourcePath);
+    MimeType contentType = SystemToolbox::AutodetectMimeType(resourcePath);
 
     try
     {
       const void* buffer = EmbeddedResources::GetDirectoryResourceBuffer(resourceId_, resourcePath.c_str());
       size_t size = EmbeddedResources::GetDirectoryResourceSize(resourceId_, resourcePath.c_str());
 
-      output.SetContentType(contentType.c_str());
+      output.SetContentType(contentType);
       output.Answer(buffer, size);
     }
     catch (OrthancException&)
