@@ -61,11 +61,6 @@ namespace Orthanc
     static void CompileGetArguments(IHttpHandler::Arguments& compiled,
                                     const IHttpHandler::GetArguments& source);
 
-    static bool SimpleGet(std::string& result,
-                          IHttpHandler& handler,
-                          RequestOrigin origin,
-                          const std::string& uri);
-
     static bool SimpleGet(std::string& result,
                           IHttpHandler& handler,
                           RequestOrigin origin,
@@ -76,18 +71,21 @@ namespace Orthanc
                            IHttpHandler& handler,
                            RequestOrigin origin,
                            const std::string& uri,
-                           const char* bodyData,
-                           size_t bodySize);
+                           const void* bodyData,
+                           size_t bodySize,
+                          const IHttpHandler::Arguments& httpHeaders);
 
     static bool SimplePut(std::string& result,
                           IHttpHandler& handler,
                           RequestOrigin origin,
                           const std::string& uri,
-                          const char* bodyData,
-                          size_t bodySize);
+                          const void* bodyData,
+                          size_t bodySize,
+                          const IHttpHandler::Arguments& httpHeaders);
 
     static bool SimpleDelete(IHttpHandler& handler,
                              RequestOrigin origin,
-                             const std::string& uri);
+                             const std::string& uri,
+                             const IHttpHandler::Arguments& httpHeaders);
   };
 }
