@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -66,7 +66,8 @@ namespace Orthanc
         buffer_ = malloc(size);
         if (buffer_ == NULL)
         {
-          throw OrthancException(ErrorCode_NotEnoughMemory);
+          throw OrthancException(ErrorCode_NotEnoughMemory,
+                                 "Failed to allocate an image buffer of size " + boost::lexical_cast<std::string>(width_) + "x" + boost::lexical_cast<std::string>(height_));
         }
       }
 
@@ -140,24 +141,18 @@ namespace Orthanc
     }
   }
 
-
-  ImageAccessor ImageBuffer::GetAccessor()
+  
+  void ImageBuffer::GetReadOnlyAccessor(ImageAccessor& accessor)
   {
     Allocate();
-
-    ImageAccessor accessor;
-    accessor.AssignWritable(format_, width_, height_, pitch_, buffer_);
-    return accessor;
-  }
-
-
-  ImageAccessor ImageBuffer::GetConstAccessor()
-  {
-    Allocate();
-
-    ImageAccessor accessor;
     accessor.AssignReadOnly(format_, width_, height_, pitch_, buffer_);
-    return accessor;
+  }
+  
+
+  void ImageBuffer::GetWriteableAccessor(ImageAccessor& accessor)
+  {
+    Allocate();
+    accessor.AssignWritable(format_, width_, height_, pitch_, buffer_);
   }
 
 
