@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -50,8 +50,14 @@ namespace Orthanc
 
     void CheckAlreadyExecuted();
 
+  protected:
     void ExecuteInternal(int numOutputs);
 
+    lua_State* GetState()
+    {
+      return context_.lua_;
+    }
+
   public:
     LuaFunctionCall(LuaContext& context,
                     const char* functionName);
