@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -43,6 +43,7 @@
 
 #include "Enumerations.h"
 
+#include <map>
 #include <vector>
 #include <string>
 #include <stdint.h>
@@ -58,7 +59,8 @@ namespace Orthanc
     ServerBarrierEvent ServerBarrier();
 
     void ReadFile(std::string& content,
-                  const std::string& path);
+                  const std::string& path,
+                  bool log = true);
 
     bool ReadHeader(std::string& header,
                     const std::string& path,
@@ -93,11 +95,19 @@ namespace Orthanc
     FILE* OpenFile(const std::string& path,
                    FileMode mode);
 
-    std::string GenerateUuid();
-
-    std::string GetNowIsoString();
+    std::string GetNowIsoString(bool utc);
 
     void GetNowDicom(std::string& date,
-                     std::string& time);
+                     std::string& time,
+                     bool utc);
+
+    unsigned int GetHardwareConcurrency();
+
+    MimeType AutodetectMimeType(const std::string& path);
+
+    void GetEnvironmentVariables(std::map<std::string, std::string>& env);
+
+    std::string InterpretRelativePath(const std::string& baseDirectory,
+                                      const std::string& relativePath);
   }
 }
