@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -68,6 +68,10 @@
  **/
 
 
+#if ORTHANC_ENABLE_PUGIXML == 1
+#  include <pugixml.hpp>
+#endif
+
 
 namespace Orthanc
 {
@@ -79,6 +83,24 @@ namespace Orthanc
 
   namespace Toolbox
   {
+    class LinesIterator
+    {
+    private:
+      const std::string& content_;
+      size_t             lineStart_;
+      size_t             lineEnd_;
+
+      void FindEndOfLine();
+  
+    public:
+      LinesIterator(const std::string& content);
+  
+      bool GetLine(std::string& target) const;
+
+      void Next();
+    };
+    
+    
     void ToUpperCase(std::string& s);  // Inplace version
 
     void ToLowerCase(std::string& s);  // Inplace version
@@ -99,8 +121,6 @@ namespace Orthanc
     bool IsChildUri(const UriComponents& baseUri,
                     const UriComponents& testedUri);
 
-    std::string AutodetectMimeType(const std::string& path);
-
     std::string FlattenUri(const UriComponents& components,
                            size_t fromLevel = 0);
 
@@ -120,7 +140,7 @@ namespace Orthanc
                      const void* data,
                      size_t size);
 
-    bool IsSHA1(const char* str,
+    bool IsSHA1(const void* str,
                 size_t size);
 
     bool IsSHA1(const std::string& s);
@@ -143,7 +163,8 @@ namespace Orthanc
 
 #if ORTHANC_ENABLE_LOCALE == 1
     std::string ConvertToUtf8(const std::string& source,
-                              Encoding sourceEncoding);
+                              Encoding sourceEncoding,
+                              bool hasCodeExtensions);
 
     std::string ConvertFromUtf8(const std::string& source,
                                 Encoding targetEncoding);
@@ -152,6 +173,8 @@ namespace Orthanc
     bool IsAsciiString(const void* data,
                        size_t size);
 
+    bool IsAsciiString(const std::string& s);
+
     std::string ConvertToAscii(const std::string& source);
 
     std::string StripSpaces(const std::string& source);
@@ -174,6 +197,11 @@ namespace Orthanc
                    const std::string& arrayElement = "item");
 #endif
 
+#if ORTHANC_ENABLE_PUGIXML == 1
+    void XmlToString(std::string& target,
+                     const pugi::xml_document& source);
+#endif
+
     bool IsInteger(const std::string& str);
 
     void CopyJsonWithoutComments(Json::Value& target,
@@ -212,5 +240,43 @@ namespace Orthanc
 
     std::string ToUpperCaseWithAccents(const std::string& source);
 #endif
+
+    void InitializeOpenSsl();
+    
+    void FinalizeOpenSsl();
+
+    std::string GenerateUuid();
+
+    std::string SubstituteVariables(const std::string& source,
+                                    const std::map<std::string, std::string>& dictionary);
+
+    void RemoveIso2022EscapeSequences(std::string& dest,
+                                      const std::string& src);
+
+    void Utf8ToUnicodeCharacter(uint32_t& unicode,
+                                size_t& utf8Length,
+                                const std::string& utf8,
+                                size_t position);
   }
 }
+
+
+
+
+/**
+ * The plain C, opaque data structure "OrthancLinesIterator" is a thin
+ * wrapper around Orthanc::Toolbox::LinesIterator, and is only used by
+ * "../Resources/WebAssembly/dcdict.cc", in order to avoid code
+ * duplication
+ **/
+
+struct OrthancLinesIterator;
+
+OrthancLinesIterator* OrthancLinesIterator_Create(const std::string& content);
+
+bool OrthancLinesIterator_GetLine(std::string& target,
+                                  const OrthancLinesIterator* iterator);
+
+void OrthancLinesIterator_Next(OrthancLinesIterator* iterator);
+
+void OrthancLinesIterator_Free(OrthancLinesIterator* iterator);
