@@ -14,34 +14,38 @@ function GenerateDicomDate(days)
 
 
 $('#query-retrieve').live('pagebeforeshow', function() {
+  var targetDate;
+
   $.ajax({
     url: '../modalities',
     dataType: 'json',
     async: false,
     cache: false,
     success: function(modalities) {
-      var target = $('#qr-server');
-      $('option', target).remove();
+      var targetServer = $('#qr-server');
+      var option;
+
+      $('option', targetServer).remove();
 
       for (var i = 0; i < modalities.length; i++) {
-        var option = $('<option>').text(modalities[i]);
-        target.append(option);
+        option = $('<option>').text(modalities[i]);
+        targetServer.append(option);
       }
 
-      target.selectmenu('refresh');
+      targetServer.selectmenu('refresh');
     }
   });
 
-  var target = $('#qr-date');
-  $('option', target).remove();
-  target.append($('<option>').attr('value', '*').text('Any date'));
-  target.append($('<option>').attr('value', GenerateDicomDate(0)).text('Today'));
-  target.append($('<option>').attr('value', GenerateDicomDate(-1)).text('Yesterday'));
-  target.append($('<option>').attr('value', GenerateDicomDate(-7) + '-').text('Last 7 days'));
-  target.append($('<option>').attr('value', GenerateDicomDate(-31) + '-').text('Last 31 days'));
-  target.append($('<option>').attr('value', GenerateDicomDate(-31 * 3) + '-').text('Last 3 months'));
-  target.append($('<option>').attr('value', GenerateDicomDate(-365) + '-').text('Last year'));
-  target.selectmenu('refresh');
+  targetDate = $('#qr-date');
+  $('option', targetDate).remove();
+  targetDate.append($('<option>').attr('value', '*').text('Any date'));
+  targetDate.append($('<option>').attr('value', GenerateDicomDate(0)).text('Today'));
+  targetDate.append($('<option>').attr('value', GenerateDicomDate(-1)).text('Yesterday'));
+  targetDate.append($('<option>').attr('value', GenerateDicomDate(-7) + '-').text('Last 7 days'));
+  targetDate.append($('<option>').attr('value', GenerateDicomDate(-31) + '-').text('Last 31 days'));
+  targetDate.append($('<option>').attr('value', GenerateDicomDate(-31 * 3) + '-').text('Last 3 months'));
+  targetDate.append($('<option>').attr('value', GenerateDicomDate(-365) + '-').text('Last year'));
+  targetDate.selectmenu('refresh');
 });
 
 
@@ -75,23 +79,26 @@ $('#qr-echo').live('click', function() {
 
 
 $('#qr-submit').live('click', function() {
-  var query = {
+  var query, server, modalities, field;
+
+  query = {
     'Level' : 'Study',
     'Query' : {
-      'AccessionNumber' : '*',
-      'PatientBirthDate' : '*',
-      'PatientID' : '*',
-      'PatientName' : '*',
-      'PatientSex' : '*',
+      'AccessionNumber' : '',
+      'PatientBirthDate' : '',
+      'PatientID' : '',
+      'PatientName' : '',
+      'PatientSex' : '',
       'StudyDate' : $('#qr-date').val(),
       'StudyDescription' : '*'
     }
   };
 
-  var field = $('#qr-fields input:checked').val();
+  modalities = '';
+
+  field = $('#qr-fields input:checked').val();
   query['Query'][field] = $('#qr-value').val().toUpperCase();
 
-  var modalities = '';
   $('#qr-modalities input:checked').each(function() {
     var s = $(this).attr('name');
     if (modalities == '')
@@ -105,7 +112,7 @@ $('#qr-submit').live('click', function() {
   }
 
 
-  var server = $('#qr-server').val();
+  server = $('#qr-server').val();
   $.ajax({
     url: '../modalities/' + server + '/query',
     type: 'POST', 
@@ -129,10 +136,12 @@ $('#qr-submit').live('click', function() {
 
 
 $('#query-retrieve-2').live('pagebeforeshow', function() {
+  var pageData, uri;
+  
   if ($.mobile.pageData) {
-    var pageData = DeepCopy($.mobile.pageData);
+    pageData = DeepCopy($.mobile.pageData);
 
-    var uri = '../queries/' + pageData.uuid + '/answers';
+    uri = '../queries/' + pageData.uuid + '/answers';
 
     $.ajax({
       url: uri,
@@ -187,10 +196,12 @@ $('#query-retrieve-2').live('pagebeforeshow', function() {
 
 
 $('#query-retrieve-3').live('pagebeforeshow', function() {
-  if ($.mobile.pageData) {
-    var pageData = DeepCopy($.mobile.pageData);
+  var pageData, query;
 
-    var query = {
+  if ($.mobile.pageData) {
+    pageData = DeepCopy($.mobile.pageData);
+
+    query = {
       'Level' : 'Series',
       'Query' : {
         'Modality' : '*',
@@ -264,6 +275,8 @@ $('#query-retrieve-3').live('pagebeforeshow', function() {
 
 
 $('#query-retrieve-4').live('pagebeforeshow', function() {
+  var pageData, uri;
+  
   if ($.mobile.pageData) {
     var pageData = DeepCopy($.mobile.pageData);
     var uri = '../queries/' + pageData.query + '/answers/' + pageData.answer + '/retrieve';
@@ -277,9 +290,11 @@ $('#query-retrieve-4').live('pagebeforeshow', function() {
         $('#retrieve-target').val(system['DicomAet']);
 
         $('#retrieve-form').submit(function(event) {
+          var aet;
+
           event.preventDefault();
 
-          var aet = $('#retrieve-target').val();
+          aet = $('#retrieve-target').val();
           if (aet.length == 0) {
             aet = system['DicomAet'];
           }
