@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2018 Osimis S.A., Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,7 +34,7 @@
 #include "PrecompiledHeadersServer.h"
 #include "LuaScripting.h"
 
-#include "OrthancInitialization.h"
+#include "OrthancConfiguration.h"
 #include "OrthancRestApi/OrthancRestApi.h"
 #include "ServerContext.h"
 
@@ -164,23 +164,37 @@ namespace Orthanc
         }
       }
       
-      Json::Value tags, metadata;
-      if (that.context_.GetIndex().LookupResource(tags, change_.GetPublicId(), change_.GetResourceType()) &&
-          that.context_.GetIndex().GetMetadata(metadata, change_.GetPublicId()))
+      Json::Value tags;
+      
+      if (that.context_.GetIndex().LookupResource(tags, change_.GetPublicId(), change_.GetResourceType()))
       {
-        LuaScripting::Lock lock(that);
+        std::map<MetadataType, std::string> metadata;
+        that.context_.GetIndex().GetAllMetadata(metadata, change_.GetPublicId());
+        
+        Json::Value formattedMetadata = Json::objectValue;
 
-        if (lock.GetLua().IsExistingFunction(name))
+        for (std::map<MetadataType, std::string>::const_iterator 
+               it = metadata.begin(); it != metadata.end(); ++it)
         {
-          that.InitializeJob();
+          std::string key = EnumerationToString(it->first);
+          formattedMetadata[key] = it->second;
+        }      
 
-          LuaFunctionCall call(lock.GetLua(), name);
-          call.PushString(change_.GetPublicId());
-          call.PushJson(tags["MainDicomTags"]);
-          call.PushJson(metadata);
-          call.Execute();
+        {
+          LuaScripting::Lock lock(that);
 
-          that.SubmitJob();
+          if (lock.GetLua().IsExistingFunction(name))
+          {
+            that.InitializeJob();
+
+            LuaFunctionCall call(lock.GetLua(), name);
+            call.PushString(change_.GetPublicId());
+            call.PushJson(tags["MainDicomTags"]);
+            call.PushJson(formattedMetadata);
+            call.Execute();
+
+            that.SubmitJob();
+          }
         }
       }
     }
@@ -265,9 +279,9 @@ namespace Orthanc
 
     // Check the types of the arguments
     int nArgs = lua_gettop(state);
-    if ((nArgs != 1 && nArgs != 2) || 
+    if (nArgs < 1 || nArgs > 3 || 
         !lua_isstring(state, 1) ||                 // URI
-        (nArgs == 2 && !lua_isboolean(state, 2)))  // Restrict to built-in API?
+        (nArgs >= 2 && !lua_isboolean(state, 2)))  // Restrict to built-in API?
     {
       LOG(ERROR) << "Lua: Bad parameters to RestApiGet()";
       lua_pushnil(state);
@@ -277,11 +291,14 @@ namespace Orthanc
     const char* uri = lua_tostring(state, 1);
     bool builtin = (nArgs == 2 ? lua_toboolean(state, 2) != 0 : false);
 
+    std::map<std::string, std::string> headers;
+    LuaContext::GetDictionaryArgument(headers, state, 3, true /* HTTP header key to lower case */);
+
     try
     {
       std::string result;
       if (HttpToolbox::SimpleGet(result, serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
-                                 RequestOrigin_Lua, uri))
+                                 RequestOrigin_Lua, uri, headers))
       {
         lua_pushlstring(state, result.c_str(), result.size());
         return 1;
@@ -311,10 +328,10 @@ namespace Orthanc
 
     // Check the types of the arguments
     int nArgs = lua_gettop(state);
-    if ((nArgs != 2 && nArgs != 3) || 
+    if (nArgs < 2 || nArgs > 4 || 
         !lua_isstring(state, 1) ||                 // URI
         !lua_isstring(state, 2) ||                 // Body
-        (nArgs == 3 && !lua_isboolean(state, 3)))  // Restrict to built-in API?
+        (nArgs >= 3 && !lua_isboolean(state, 3)))  // Restrict to built-in API?
     {
       LOG(ERROR) << "Lua: Bad parameters to " << (isPost ? "RestApiPost()" : "RestApiPut()");
       lua_pushnil(state);
@@ -326,14 +343,17 @@ namespace Orthanc
     const char* bodyData = lua_tolstring(state, 2, &bodySize);
     bool builtin = (nArgs == 3 ? lua_toboolean(state, 3) != 0 : false);
 
+    std::map<std::string, std::string> headers;
+    LuaContext::GetDictionaryArgument(headers, state, 4, true /* HTTP header key to lower case */);
+        
     try
     {
       std::string result;
       if (isPost ?
           HttpToolbox::SimplePost(result, serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
-                                  RequestOrigin_Lua, uri, bodyData, bodySize) :
+                                  RequestOrigin_Lua, uri, bodyData, bodySize, headers) :
           HttpToolbox::SimplePut(result, serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
-                                 RequestOrigin_Lua, uri, bodyData, bodySize))
+                                 RequestOrigin_Lua, uri, bodyData, bodySize, headers))
       {
         lua_pushlstring(state, result.c_str(), result.size());
         return 1;
@@ -377,9 +397,9 @@ namespace Orthanc
 
     // Check the types of the arguments
     int nArgs = lua_gettop(state);
-    if ((nArgs != 1 && nArgs != 2) || 
+    if (nArgs < 1 || nArgs > 3 ||
         !lua_isstring(state, 1) ||                 // URI
-        (nArgs == 2 && !lua_isboolean(state, 2)))  // Restrict to built-in API?
+        (nArgs >= 2 && !lua_isboolean(state, 2)))  // Restrict to built-in API?
     {
       LOG(ERROR) << "Lua: Bad parameters to RestApiDelete()";
       lua_pushnil(state);
@@ -389,10 +409,13 @@ namespace Orthanc
     const char* uri = lua_tostring(state, 1);
     bool builtin = (nArgs == 2 ? lua_toboolean(state, 2) != 0 : false);
 
+    std::map<std::string, std::string> headers;
+    LuaContext::GetDictionaryArgument(headers, state, 3, true /* HTTP header key to lower case */);
+    
     try
     {
       if (HttpToolbox::SimpleDelete(serverContext->GetHttpHandler().RestrictToOrthancRestApi(builtin), 
-                                    RequestOrigin_Lua, uri))
+                                    RequestOrigin_Lua, uri, headers))
       {
         lua_pushboolean(state, 1);
         return 1;
@@ -414,7 +437,11 @@ namespace Orthanc
   int LuaScripting::GetOrthancConfiguration(lua_State *state)
   {
     Json::Value configuration;
-    Configuration::GetConfiguration(configuration);
+
+    {
+      OrthancConfiguration::ReaderLock lock;
+      configuration = lock.GetJson();
+    }
 
     LuaContext::GetLuaContext(state).PushJson(configuration);
 
@@ -445,7 +472,12 @@ namespace Orthanc
       }
 
       std::string name = parameters["Modality"].asString();
-      RemoteModalityParameters modality = Configuration::GetModalityUsingSymbolicName(name);
+      RemoteModalityParameters modality;
+
+      {
+        OrthancConfiguration::ReaderLock configLock;
+        modality = configLock.GetConfiguration().GetModalityUsingSymbolicName(name);
+      }
 
       // This is not a C-MOVE: No need to call "StoreScuCommand::SetMoveOriginator()"
       return lock.AddStoreScuOperation(localAet, modality);
@@ -453,12 +485,19 @@ namespace Orthanc
 
     if (operation == "store-peer")
     {
+      OrthancConfiguration::ReaderLock configLock;
       std::string name = parameters["Peer"].asString();
 
       WebServiceParameters peer;
-      Configuration::GetOrthancPeer(peer, name);
-
-      return lock.AddStorePeerOperation(peer);
+      if (configLock.GetConfiguration().LookupOrthancPeer(peer, name))
+      {
+        return lock.AddStorePeerOperation(peer);
+      }
+      else
+      {
+        throw OrthancException(ErrorCode_UnknownResource,
+                               "No peer with symbolic name: " + name);
+      }
     }
 
     if (operation == "modify")
@@ -736,17 +775,19 @@ namespace Orthanc
 
   void LuaScripting::LoadGlobalConfiguration()
   {
+    OrthancConfiguration::ReaderLock configLock;
+
     lua_.Execute(Orthanc::EmbeddedResources::LUA_TOOLBOX);
 
     std::list<std::string> luaScripts;
-    Configuration::GetGlobalListOfStringsParameter(luaScripts, "LuaScripts");
+    configLock.GetConfiguration().GetListOfStringsParameter(luaScripts, "LuaScripts");
 
     LuaScripting::Lock lock(*this);
 
     for (std::list<std::string>::const_iterator
            it = luaScripts.begin(); it != luaScripts.end(); ++it)
     {
-      std::string path = Configuration::InterpretStringParameterAsPath(*it);
+      std::string path = configLock.GetConfiguration().InterpretStringParameterAsPath(*it);
       LOG(INFO) << "Installing the Lua scripts from: " << path;
       std::string script;
       SystemToolbox::ReadFile(script, path);
