@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,10 +34,14 @@
 #include "PrecompiledHeadersServer.h"
 #include "OrthancMoveRequestHandler.h"
 
-#include "OrthancInitialization.h"
 #include "../../Core/DicomParsing/FromDcmtkBridge.h"
 #include "../Core/DicomFormat/DicomArray.h"
 #include "../Core/Logging.h"
+#include "../Core/MetricsRegistry.h"
+#include "OrthancConfiguration.h"
+#include "ServerContext.h"
+#include "ServerJobs/DicomModalityStoreJob.h"
+
 
 namespace Orthanc
 {
@@ -45,7 +49,7 @@ namespace Orthanc
   {
     // Anonymous namespace to avoid clashes between compilation modules
 
-    class OrthancMoveRequestIterator : public IMoveRequestIterator
+    class SynchronousMove : public IMoveRequestIterator
     {
     private:
       ServerContext& context_;
@@ -55,31 +59,39 @@ namespace Orthanc
       RemoteModalityParameters remote_;
       std::string originatorAet_;
       uint16_t originatorId_;
+      std::auto_ptr<DicomUserConnection> connection_;
 
     public:
-      OrthancMoveRequestIterator(ServerContext& context,
-                                 const std::string& aet,
-                                 const std::string& publicId,
-                                 const std::string& originatorAet,
-                                 uint16_t originatorId) :
+      SynchronousMove(ServerContext& context,
+                      const std::string& targetAet,
+                      const std::vector<std::string>& publicIds,
+                      const std::string& originatorAet,
+                      uint16_t originatorId) :
         context_(context),
         localAet_(context.GetDefaultLocalApplicationEntityTitle()),
         position_(0),
         originatorAet_(originatorAet),
         originatorId_(originatorId)
       {
-        LOG(INFO) << "Sending resource " << publicId << " to modality \"" << aet << "\"";
-
-        std::list<std::string> tmp;
-        context_.GetIndex().GetChildInstances(tmp, publicId);
-
-        instances_.reserve(tmp.size());
-        for (std::list<std::string>::iterator it = tmp.begin(); it != tmp.end(); ++it)
         {
-          instances_.push_back(*it);
+          OrthancConfiguration::ReaderLock lock;
+          remote_ = lock.GetConfiguration().GetModalityUsingAet(targetAet);
         }
 
-        remote_ = Configuration::GetModalityUsingAet(aet);
+        for (size_t i = 0; i < publicIds.size(); i++)
+        {
+          LOG(INFO) << "Sending resource " << publicIds[i] << " to modality \""
+                    << targetAet << "\" in synchronous mode";
+
+          std::list<std::string> tmp;
+          context_.GetIndex().GetChildInstances(tmp, publicIds[i]);
+
+          instances_.reserve(tmp.size());
+          for (std::list<std::string>::iterator it = tmp.begin(); it != tmp.end(); ++it)
+          {
+            instances_.push_back(*it);
+          }
+        }
       }
 
       virtual unsigned int GetSubOperationCount() const
@@ -99,21 +111,96 @@ namespace Orthanc
         std::string dicom;
         context_.ReadDicom(dicom, id);
 
+        if (connection_.get() == NULL)
         {
-          ReusableDicomUserConnection::Locker locker
-            (context_.GetReusableDicomUserConnection(), localAet_, remote_);
-          locker.GetConnection().Store(dicom, originatorAet_, originatorId_);
+          connection_.reset(new DicomUserConnection(localAet_, remote_));
         }
 
+        connection_->Store(dicom, originatorAet_, originatorId_);
+
+        return Status_Success;
+      }
+    };
+
+
+    class AsynchronousMove : public IMoveRequestIterator
+    {
+    private:
+      ServerContext&                        context_;
+      std::auto_ptr<DicomModalityStoreJob>  job_;
+      size_t                                position_;
+      size_t                                countInstances_;
+      
+    public:
+      AsynchronousMove(ServerContext& context,
+                       const std::string& targetAet,
+                       const std::vector<std::string>& publicIds,
+                       const std::string& originatorAet,
+                       uint16_t originatorId) :
+        context_(context),
+        job_(new DicomModalityStoreJob(context)),
+        position_(0)
+      {
+        job_->SetDescription("C-MOVE");
+        job_->SetPermissive(true);
+        job_->SetLocalAet(context.GetDefaultLocalApplicationEntityTitle());
+
+        {
+          OrthancConfiguration::ReaderLock lock;
+          job_->SetRemoteModality(lock.GetConfiguration().GetModalityUsingAet(targetAet));
+        }
+
+        if (originatorId != 0)
+        {
+          job_->SetMoveOriginator(originatorAet, originatorId);
+        }
+
+        for (size_t i = 0; i < publicIds.size(); i++)
+        {
+          LOG(INFO) << "Sending resource " << publicIds[i] << " to modality \""
+                    << targetAet << "\" in asynchronous mode";
+
+          std::list<std::string> tmp;
+          context_.GetIndex().GetChildInstances(tmp, publicIds[i]);
+
+          countInstances_ = tmp.size();
+
+          job_->Reserve(job_->GetCommandsCount() + tmp.size());
+
+          for (std::list<std::string>::iterator it = tmp.begin(); it != tmp.end(); ++it)
+          {
+            job_->AddInstance(*it);
+          }
+        }
+      }
+
+      virtual unsigned int GetSubOperationCount() const
+      {
+        return countInstances_;
+      }
+
+      virtual Status DoNext()
+      {
+        if (position_ >= countInstances_)
+        {
+          return Status_Failure;
+        }
+        
+        if (position_ == 0)
+        {
+          context_.GetJobsEngine().GetRegistry().Submit(job_.release(), 0 /* priority */);
+        }
+        
+        position_ ++;
         return Status_Success;
       }
     };
   }
 
 
-  bool OrthancMoveRequestHandler::LookupIdentifier(std::string& publicId,
-                                                   ResourceType level,
-                                                   const DicomMap& input)
+  bool OrthancMoveRequestHandler::LookupIdentifiers(std::vector<std::string>& publicIds,
+                                                    ResourceType level,
+                                                    const DicomMap& input)
   {
     DicomTag tag(0, 0);   // Dummy initialization
 
@@ -151,20 +238,41 @@ namespace Orthanc
     {
       return false;
     }
-
-    const std::string& content = value.GetContent();
-
-    std::list<std::string> ids;
-    context_.GetIndex().LookupIdentifierExact(ids, level, tag, content);
-
-    if (ids.size() != 1)
+    else
     {
-      return false;
+      const std::string& content = value.GetContent();
+      context_.GetIndex().LookupIdentifierExact(publicIds, level, tag, content);
+      return true;
+    }
+  }
+
+
+  static IMoveRequestIterator* CreateIterator(ServerContext& context,
+                                              const std::string& targetAet,
+                                              const std::vector<std::string>& publicIds,
+                                              const std::string& originatorAet,
+                                              uint16_t originatorId)
+  {
+    if (publicIds.empty())
+    {
+      throw OrthancException(ErrorCode_BadRequest,
+                             "C-MOVE request matching no resource stored in Orthanc");
+    }
+    
+    bool synchronous;
+
+    {
+      OrthancConfiguration::ReaderLock lock;
+      synchronous = lock.GetConfiguration().GetBooleanParameter("SynchronousCMove", true);
+    }
+
+    if (synchronous)
+    {
+      return new SynchronousMove(context, targetAet, publicIds, originatorAet, originatorId);
     }
     else
     {
-      publicId = ids.front();
-      return true;
+      return new AsynchronousMove(context, targetAet, publicIds, originatorAet, originatorId);
     }
   }
 
@@ -176,6 +284,8 @@ namespace Orthanc
                                                           const std::string& calledAet,
                                                           uint16_t originatorId)
   {
+    MetricsRegistry::Timer timer(context_.GetMetricsRegistry(), "orthanc_move_scp_duration_ms");
+
     LOG(WARNING) << "Move-SCU request received for AET \"" << targetAet << "\"";
 
     {
@@ -191,7 +301,6 @@ namespace Orthanc
       }
     }
 
-
     /**
      * Retrieve the query level.
      **/
@@ -208,19 +317,19 @@ namespace Orthanc
       // query level: Start from the instance level, going up to the
       // patient level until a valid DICOM identifier is found.
 
-      std::string publicId;
+      std::vector<std::string> publicIds;
 
-      if (LookupIdentifier(publicId, ResourceType_Instance, input) ||
-          LookupIdentifier(publicId, ResourceType_Series, input) ||
-          LookupIdentifier(publicId, ResourceType_Study, input) ||
-          LookupIdentifier(publicId, ResourceType_Patient, input))
+      if (LookupIdentifiers(publicIds, ResourceType_Instance, input) ||
+          LookupIdentifiers(publicIds, ResourceType_Series, input) ||
+          LookupIdentifiers(publicIds, ResourceType_Study, input) ||
+          LookupIdentifiers(publicIds, ResourceType_Patient, input))
       {
-        return new OrthancMoveRequestIterator(context_, targetAet, publicId, originatorAet, originatorId);
+        return CreateIterator(context_, targetAet, publicIds, originatorAet, originatorId);
       }
       else
       {
         // No identifier is present in the request.
-        throw OrthancException(ErrorCode_BadRequest);
+        throw OrthancException(ErrorCode_BadRequest, "Invalid fields in a C-MOVE request");
       }
     }
 
@@ -232,15 +341,15 @@ namespace Orthanc
      * Lookup for the resource to be sent.
      **/
 
-    std::string publicId;
+    std::vector<std::string> publicIds;
 
-    if (LookupIdentifier(publicId, level, input))
+    if (LookupIdentifiers(publicIds, level, input))
     {
-      return new OrthancMoveRequestIterator(context_, targetAet, publicId, originatorAet, originatorId);
+      return CreateIterator(context_, targetAet, publicIds, originatorAet, originatorId);
     }
     else
     {
-      throw OrthancException(ErrorCode_BadRequest);
+      throw OrthancException(ErrorCode_BadRequest, "Invalid fields in a C-MOVE request");
     }
   }
 }
