@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,8 +33,10 @@
 
 #pragma once
 
-#include "../../Core/RestApi/RestApi.h"
 #include "../../Core/DicomParsing/DicomModification.h"
+#include "../../Core/JobsEngine/SetOfCommandsJob.h"
+#include "../../Core/MetricsRegistry.h"
+#include "../../Core/RestApi/RestApi.h"
 #include "../ServerEnumerations.h"
 
 #include <set>
@@ -43,6 +45,7 @@ namespace Orthanc
 {
   class ServerContext;
   class ServerIndex;
+  class DicomInstanceToStore;
 
   class OrthancRestApi : public RestApi
   {
@@ -50,9 +53,10 @@ namespace Orthanc
     typedef std::set<std::string> SetOfStrings;
 
   private:
-    ServerContext& context_;
-    bool leaveBarrier_;
-    bool resetRequestReceived_;
+    ServerContext&                  context_;
+    bool                            leaveBarrier_;
+    bool                            resetRequestReceived_;
+    MetricsRegistry::SharedMetrics  activeRequests_;
 
     void RegisterSystem();
 
@@ -73,6 +77,17 @@ namespace Orthanc
   public:
     OrthancRestApi(ServerContext& context);
 
+    virtual bool Handle(HttpOutput& output,
+                        RequestOrigin origin,
+                        const char* remoteIp,
+                        const char* username,
+                        HttpMethod method,
+                        const UriComponents& uri,
+                        const Arguments& headers,
+                        const GetArguments& getArguments,
+                        const void* bodyData,
+                        size_t bodySize) ORTHANC_OVERRIDE;
+
     const bool& LeaveBarrierFlag() const
     {
       return leaveBarrier_;
@@ -92,12 +107,34 @@ namespace Orthanc
 
     static ServerIndex& GetIndex(RestApiCall& call);
 
+    void AnswerStoredInstance(RestApiPostCall& call,
+                              DicomInstanceToStore& instance,
+                              StoreStatus status) const;
+
     void AnswerStoredResource(RestApiPostCall& call,
                               const std::string& publicId,
                               ResourceType resourceType,
                               StoreStatus status) const;
 
-    static bool ParseModifyRequest(DicomModification& target,
-                                   const Json::Value& request);
+    static bool IsSynchronousJobRequest(bool isDefaultSynchronous,
+                                        const Json::Value& body);
+    
+    static unsigned int GetJobRequestPriority(const Json::Value& body);
+    
+    static void SubmitGenericJob(RestApiOutput& output,
+                                 ServerContext& context,
+                                 IJob* job,
+                                 bool synchronous,
+                                 int priority);
+    
+    void SubmitGenericJob(RestApiPostCall& call,
+                          IJob* job,
+                          bool isDefaultSynchronous,
+                          const Json::Value& body) const;
+
+    void SubmitCommandsJob(RestApiPostCall& call,
+                           SetOfCommandsJob* job,
+                           bool isDefaultSynchronous,
+                           const Json::Value& body) const;
   };
 }
