@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2018 Osimis S.A., Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,10 +34,12 @@
 #include "PrecompiledHeadersServer.h"
 #include "QueryRetrieveHandler.h"
 
-#include "OrthancInitialization.h"
+#include "OrthancConfiguration.h"
 
 #include "../Core/DicomParsing/FromDcmtkBridge.h"
 #include "../Core/Logging.h"
+#include "LuaScripting.h"
+#include "ServerContext.h"
 
 
 namespace Orthanc
@@ -60,38 +62,10 @@ namespace Orthanc
   }
 
 
-  static void FixQueryBuiltin(DicomMap& query,
-                              ModalityManufacturer manufacturer)
-  {
-    /**
-     * Introduce patches for specific manufacturers below.
-     **/
-
-    switch (manufacturer)
-    {
-      default:
-        break;
-    }
-  }
-
-
   void QueryRetrieveHandler::Invalidate()
   {
     done_ = false;
     answers_.Clear();
-    connection_.reset(NULL);
-  }
-
-
-  DicomUserConnection& QueryRetrieveHandler::GetConnection()
-  {
-    if (connection_.get() == NULL)
-    {
-      connection_.reset(new DicomUserConnection(localAet_, modality_));
-      connection_->Open();
-    }
-
-    return *connection_;
   }
 
 
@@ -102,12 +76,15 @@ namespace Orthanc
       // Firstly, fix the content of the query for specific manufacturers
       DicomMap fixed;
       fixed.Assign(query_);
-      FixQueryBuiltin(fixed, modality_.GetManufacturer());
 
       // Secondly, possibly fix the query with the user-provider Lua callback
       FixQueryLua(fixed, context_, modality_.GetApplicationEntityTitle()); 
 
-      GetConnection().Find(answers_, level_, fixed);
+      {
+        DicomUserConnection connection(localAet_, modality_);
+        connection.Open();
+        connection.Find(answers_, level_, fixed, findNormalized_);
+      }
 
       done_ = true;
     }
@@ -119,7 +96,8 @@ namespace Orthanc
     localAet_(context.GetDefaultLocalApplicationEntityTitle()),
     done_(false),
     level_(ResourceType_Study),
-    answers_(false)
+    answers_(false),
+    findNormalized_(true)
   {
   }
 
@@ -128,7 +106,11 @@ namespace Orthanc
   {
     Invalidate();
     modalityName_ = symbolicName;
-    Configuration::GetDicomModalityUsingSymbolicName(modality_, symbolicName);
+
+    {
+      OrthancConfiguration::ReaderLock lock;
+      lock.GetConfiguration().GetDicomModalityUsingSymbolicName(modality_, symbolicName);
+    }
   }
 
 
@@ -147,7 +129,25 @@ namespace Orthanc
   }
 
 
-  size_t QueryRetrieveHandler::GetAnswerCount()
+  void QueryRetrieveHandler::CopyStringTag(const DicomMap& from,
+                                           const DicomTag& tag)
+  {
+    const DicomValue* value = from.TestAndGetValue(tag);
+
+    if (value == NULL ||
+        value->IsNull() ||
+        value->IsBinary())
+    {
+      throw OrthancException(ErrorCode_InexistentTag);
+    }
+    else
+    {
+      SetQuery(tag, value->GetContent());
+    }
+  }
+
+
+  size_t QueryRetrieveHandler::GetAnswersCount()
   {
     Run();
     return answers_.GetSize();
@@ -161,21 +161,10 @@ namespace Orthanc
     answers_.GetAnswer(i).ExtractDicomSummary(target);
   }
 
-
-  void QueryRetrieveHandler::Retrieve(const std::string& target,
-                                      size_t i)
+  
+  void QueryRetrieveHandler::SetFindNormalized(bool normalized)
   {
-    DicomMap map;
-    GetAnswer(map, i);
-    GetConnection().Move(target, map);
-  }
-
-
-  void QueryRetrieveHandler::Retrieve(const std::string& target)
-  {
-    for (size_t i = 0; i < GetAnswerCount(); i++)
-    {
-      Retrieve(target, i);
-    }
+    Invalidate();
+    findNormalized_ = normalized;
   }
 }
