@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,35 +33,43 @@
 
 #pragma once
 
-#include "ServerContext.h"
+#include "../Core/DicomNetworking/DicomFindAnswers.h"
+#include "../Core/DicomNetworking/RemoteModalityParameters.h"
 
 namespace Orthanc
 {
+  class ServerContext;
+  
   class QueryRetrieveHandler : public IDynamicObject
   {
   private:
     ServerContext&             context_;
-    const std::string&         localAet_;
+    std::string                localAet_;
     bool                       done_;
     RemoteModalityParameters   modality_;
     ResourceType               level_;
     DicomMap                   query_;
     DicomFindAnswers           answers_;
     std::string                modalityName_;
+    bool                       findNormalized_;
 
     void Invalidate();
 
-
   public:
     QueryRetrieveHandler(ServerContext& context);
 
     void SetModality(const std::string& symbolicName);
 
-    const RemoteModalityParameters& GetModality() const
+    const RemoteModalityParameters& GetRemoteModality() const
     {
       return modality_;
     }
 
+    const std::string& GetLocalAet() const
+    {
+      return localAet_;
+    }
+
     const std::string& GetModalitySymbolicName() const
     {
       return modalityName_;
@@ -82,16 +90,21 @@ namespace Orthanc
       return query_;
     }
 
+    void CopyStringTag(const DicomMap& from,
+                       const DicomTag& tag);
+
     void Run();
 
-    size_t GetAnswerCount();
+    size_t GetAnswersCount();
 
     void GetAnswer(DicomMap& target,
                    size_t i);
 
-    void Retrieve(const std::string& target,
-                  size_t i);
+    bool IsFindNormalized() const
+    {
+      return findNormalized_;
+    }
 
-    void Retrieve(const std::string& target);
+    void SetFindNormalized(bool normalized);
   };
 }
