@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2018 Osimis S.A., Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,28 +33,28 @@
 
 #pragma once
 
-#include "DicomInstanceToStore.h"
 #include "IServerListener.h"
 #include "LuaScripting.h"
 #include "OrthancHttpHandler.h"
 #include "ServerIndex.h"
 
 #include "../Core/Cache/MemoryCache.h"
-#include "../Core/Cache/SharedArchive.h"
-#include "../Core/DicomParsing/ParsedDicomFile.h"
-#include "../Core/FileStorage/IStorageArea.h"
-#include "../Core/JobsEngine/JobsEngine.h"
-#include "../Core/JobsEngine/SetOfInstancesJob.h"
-#include "../Core/MultiThreading/SharedMessageQueue.h"
-#include "../Core/RestApi/RestApiOutput.h"
-#include "../Plugins/Engine/OrthancPlugins.h"
-
-#include <boost/filesystem.hpp>
-#include <boost/thread.hpp>
 
 
 namespace Orthanc
 {
+  class DicomInstanceToStore;
+  class IStorageArea;
+  class JobsEngine;
+  class MetricsRegistry;
+  class OrthancPlugins;
+  class ParsedDicomFile;
+  class RestApiOutput;
+  class SetOfInstancesJob;
+  class SharedArchive;
+  class SharedMessageQueue;
+  
+  
   /**
    * This class is responsible for maintaining the storage area on the
    * filesystem (including compression), as well as the index of the
@@ -62,7 +62,34 @@ namespace Orthanc
    **/
   class ServerContext : private JobsRegistry::IObserver
   {
+  public:
+    class ILookupVisitor : public boost::noncopyable
+    {
+    public:
+      virtual ~ILookupVisitor()
+      {
+      }
+
+      virtual bool IsDicomAsJsonNeeded() const = 0;
+      
+      virtual void MarkAsComplete() = 0;
+
+      virtual void Visit(const std::string& publicId,
+                         const std::string& instanceId,
+                         const DicomMap& mainDicomTags,
+                         const Json::Value* dicomAsJson) = 0;
+    };
+    
+    
   private:
+    enum LookupMode
+    {
+      LookupMode_DatabaseOnly,
+      LookupMode_DiskOnAnswer,
+      LookupMode_DiskOnLookupAndAnswer
+    };
+
+    
     class LuaServerListener : public IServerListener
     {
     private:
@@ -143,9 +170,6 @@ namespace Orthanc
     void ReadDicomAsJsonInternal(std::string& result,
                                  const std::string& instancePublicId);
 
-    void SetupJobsEngine(bool unitTesting,
-                         bool loadJobsFromDatabase);
-
     void SaveJobsEngine();
 
     virtual void SignalJobSubmitted(const std::string& jobId);
@@ -163,12 +187,19 @@ namespace Orthanc
     DicomCacheProvider provider_;
     boost::mutex dicomCacheMutex_;
     MemoryCache dicomCache_;
-    JobsEngine jobsEngine_;
 
     LuaScripting mainLua_;
     LuaScripting filterLua_;
     LuaServerListener  luaListener_;
-
+    std::auto_ptr<SharedArchive>  mediaArchive_;
+    
+    // The "JobsEngine" must be *after* "LuaScripting", as
+    // "LuaScripting" embeds "LuaJobManager" that registers as an
+    // observer to "SequenceOfOperationsJob", whose lifetime
+    // corresponds to that of "JobsEngine". It must also be after
+    // "mediaArchive_", as jobs might access this archive.
+    JobsEngine jobsEngine_;
+    
 #if ORTHANC_ENABLE_PLUGINS == 1
     OrthancPlugins* plugins_;
 #endif
@@ -178,13 +209,19 @@ namespace Orthanc
 
     bool done_;
     bool haveJobsChanged_;
+    bool isJobsEngineUnserialized_;
     SharedMessageQueue  pendingChanges_;
     boost::thread  changeThread_;
     boost::thread  saveJobsThread_;
         
-    SharedArchive  queryRetrieveArchive_;
+    std::auto_ptr<SharedArchive>  queryRetrieveArchive_;
     std::string defaultLocalAet_;
     OrthancHttpHandler  httpHandler_;
+    bool saveJobs_;
+
+    std::auto_ptr<MetricsRegistry>  metricsRegistry_;
+    bool isHttpServerSecure_;
+    bool isExecuteLuaEnabled_;
 
   public:
     class DicomCacheLocker : public boost::noncopyable
@@ -209,10 +246,13 @@ namespace Orthanc
     ServerContext(IDatabaseWrapper& database,
                   IStorageArea& area,
                   bool unitTesting,
-                  bool loadJobsFromDatabase);
+                  size_t maxCompletedJobs);
 
     ~ServerContext();
 
+    void SetupJobsEngine(bool unitTesting,
+                         bool loadJobsFromDatabase);
+
     ServerIndex& GetIndex()
     {
       return index_;
@@ -301,7 +341,12 @@ namespace Orthanc
 
     SharedArchive& GetQueryRetrieveArchive()
     {
-      return queryRetrieveArchive_;
+      return *queryRetrieveArchive_;
+    }
+
+    SharedArchive& GetMediaArchive()
+    {
+      return *mediaArchive_;
     }
 
     const std::string& GetDefaultLocalApplicationEntityTitle() const
@@ -321,11 +366,16 @@ namespace Orthanc
 
     void Stop();
 
-    void Apply(std::list<std::string>& result,
-               const ::Orthanc::LookupResource& lookup,
+    void Apply(ILookupVisitor& visitor,
+               const DatabaseLookup& lookup,
+               ResourceType queryLevel,
                size_t since,
                size_t limit);
 
+    bool LookupOrReconstructMetadata(std::string& target,
+                                     const std::string& publicId,
+                                     MetadataType type);
+
 
     /**
      * Management of the plugins
@@ -345,5 +395,34 @@ namespace Orthanc
 
     void AddChildInstances(SetOfInstancesJob& job,
                            const std::string& publicId);
+
+    void SignalUpdatedModalities();
+
+    void SignalUpdatedPeers();
+
+    MetricsRegistry& GetMetricsRegistry()
+    {
+      return *metricsRegistry_;
+    }
+
+    void SetHttpServerSecure(bool isSecure)
+    {
+      isHttpServerSecure_ = isSecure;
+    }
+
+    bool IsHttpServerSecure() const
+    {
+      return isHttpServerSecure_;
+    }
+
+    void SetExecuteLuaEnabled(bool enabled)
+    {
+      isExecuteLuaEnabled_ = enabled;
+    }
+
+    bool IsExecuteLuaEnabled() const
+    {
+      return isExecuteLuaEnabled_;
+    }
   };
 }
