@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -59,7 +59,7 @@ namespace Orthanc
       changeType_(changeType),
       resourceType_(resourceType),
       publicId_(publicId),
-      date_(SystemToolbox::GetNowIsoString())
+      date_(SystemToolbox::GetNowIsoString(true /* use UTC time (not local time) */))
     {
     }
 
