@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,6 +37,7 @@
 #include "../Core/Logging.h"
 #include "../Core/Toolbox.h"
 #include "ServerEnumerations.h"
+#include "ServerIndex.h"
 
 #include <algorithm>
 #include <boost/lexical_cast.hpp>
@@ -95,12 +96,69 @@ namespace Orthanc
   }
 
 
+  static bool IsCloseToZero(double x)
+  {
+    return fabs(x) < 10.0 * std::numeric_limits<float>::epsilon();
+  }
+
+  
+  bool SliceOrdering::ComputeNormal(Vector& normal,
+                                    const DicomMap& dicom)
+  {
+    std::vector<float> cosines;
+
+    if (TokenizeVector(cosines, dicom, DICOM_TAG_IMAGE_ORIENTATION_PATIENT, 6))
+    {
+      assert(cosines.size() == 6);
+      normal[0] = cosines[1] * cosines[5] - cosines[2] * cosines[4];
+      normal[1] = cosines[2] * cosines[3] - cosines[0] * cosines[5];
+      normal[2] = cosines[0] * cosines[4] - cosines[1] * cosines[3];
+      return true;
+    }
+    else
+    {
+      return false;
+    }
+  }
+
+
+  bool SliceOrdering::IsParallelOrOpposite(const Vector& u,
+                                           const Vector& v)
+  {
+    // Check out "GeometryToolbox::IsParallelOrOpposite()" in Stone of
+    // Orthanc for explanations
+    const double u1 = u[0];
+    const double u2 = u[1];
+    const double u3 = u[2];
+    const double normU = sqrt(u1 * u1 + u2 * u2 + u3 * u3);
+
+    const double v1 = v[0];
+    const double v2 = v[1];
+    const double v3 = v[2];
+    const double normV = sqrt(v1 * v1 + v2 * v2 + v3 * v3);
+
+    if (IsCloseToZero(normU * normV))
+    {
+      return false;
+    }
+    else
+    {
+      const double cosAngle = (u1 * v1 + u2 * v2 + u3 * v3) / (normU * normV);
+
+      return (IsCloseToZero(cosAngle - 1.0) ||      // Close to +1: Parallel, non-opposite
+              IsCloseToZero(fabs(cosAngle) - 1.0)); // Close to -1: Parallel, opposite
+    }
+  }
+
+  
   struct SliceOrdering::Instance : public boost::noncopyable
   {
   private:
     std::string   instanceId_;
     bool          hasPosition_;
     Vector        position_;   
+    bool          hasNormal_;
+    Vector        normal_;   
     bool          hasIndexInSeries_;
     size_t        indexInSeries_;
     unsigned int  framesCount_;
@@ -141,6 +199,8 @@ namespace Orthanc
         position_[2] = tmp[2];
       }
 
+      hasNormal_ = ComputeNormal(normal_, instance);
+
       std::string s;
       hasIndexInSeries_ = false;
 
@@ -190,6 +250,17 @@ namespace Orthanc
     {
       return framesCount_;
     }
+
+    bool HasNormal() const
+    {
+      return hasNormal_;
+    }
+
+    const Vector& GetNormal() const
+    {
+      assert(hasNormal_);
+      return normal_;
+    }
   };
 
 
@@ -226,15 +297,7 @@ namespace Orthanc
       throw OrthancException(ErrorCode_UnknownResource);
     }
 
-    std::vector<float> cosines;
-    hasNormal_ = TokenizeVector(cosines, series, DICOM_TAG_IMAGE_ORIENTATION_PATIENT, 6);
-
-    if (hasNormal_)
-    {
-      normal_[0] = cosines[1] * cosines[5] - cosines[2] * cosines[4];
-      normal_[1] = cosines[2] * cosines[3] - cosines[0] * cosines[5];
-      normal_[2] = cosines[0] * cosines[4] - cosines[1] * cosines[3];
-    }
+    hasNormal_ = ComputeNormal(normal_, series);
   }
 
 
@@ -257,6 +320,7 @@ namespace Orthanc
     if (instances_.size() <= 1)
     {
       // One single instance: It is sorted by default
+      sortedInstances_ = instances_;
       return true;
     }
 
@@ -265,22 +329,32 @@ namespace Orthanc
       return false;
     }
 
+    sortedInstances_.clear();
+
+    // consider only the instances with a position and correctly oriented (if they have a normal)
     for (size_t i = 0; i < instances_.size(); i++)
     {
       assert(instances_[i] != NULL);
-      if (!instances_[i]->HasPosition())
+      if (instances_[i]->HasPosition() &&
+          (!instances_[i]->HasNormal() ||
+           IsParallelOrOpposite(instances_[i]->GetNormal(), normal_)))
       {
-        return false;
+        sortedInstances_.push_back(instances_[i]);
       }
     }
 
-    PositionComparator comparator(normal_);
-    std::sort(instances_.begin(), instances_.end(), comparator);
-
-    float a = instances_[0]->ComputeRelativePosition(normal_);
-    for (size_t i = 1; i < instances_.size(); i++)
+    if (sortedInstances_.size() == 0)
     {
-      float b = instances_[i]->ComputeRelativePosition(normal_);
+      return false;
+    }
+
+    PositionComparator comparator(normal_);
+    std::sort(sortedInstances_.begin(), sortedInstances_.end(), comparator);
+
+    float a = sortedInstances_[0]->ComputeRelativePosition(normal_);
+    for (size_t i = 1; i < sortedInstances_.size(); i++)
+    {
+      float b = sortedInstances_[i]->ComputeRelativePosition(normal_);
 
       if (std::fabs(b - a) <= 10.0f * std::numeric_limits<float>::epsilon())
       {
@@ -302,27 +376,38 @@ namespace Orthanc
     if (instances_.size() <= 1)
     {
       // One single instance: It is sorted by default
+      sortedInstances_ = instances_;
       return true;
     }
 
+    sortedInstances_.clear();
+
+    // consider only the instances with an index
     for (size_t i = 0; i < instances_.size(); i++)
     {
       assert(instances_[i] != NULL);
-      if (!instances_[i]->HasIndexInSeries())
+      if (instances_[i]->HasIndexInSeries())
       {
-        return false;
+        sortedInstances_.push_back(instances_[i]);
       }
     }
 
-    std::sort(instances_.begin(), instances_.end(), IndexInSeriesComparator);
-    
-    for (size_t i = 1; i < instances_.size(); i++)
+    if (sortedInstances_.size() == 0) // if we were not able to sort instances because none of them had an index, return all instances in a "random" order
     {
-      if (instances_[i - 1]->GetIndexInSeries() == instances_[i]->GetIndexInSeries())
+      sortedInstances_ = instances_;
+    }
+    else
+    {
+      std::sort(sortedInstances_.begin(), sortedInstances_.end(), IndexInSeriesComparator);
+
+      for (size_t i = 1; i < sortedInstances_.size(); i++)
       {
-        // The current "IndexInSeries" occurs 2 times: Not a proper ordering
-        LOG(WARNING) << "This series contains 2 slices with the same index, trying to display it anyway";
-        break;
+        if (sortedInstances_[i - 1]->GetIndexInSeries() == sortedInstances_[i]->GetIndexInSeries())
+        {
+          // The current "IndexInSeries" occurs 2 times: Not a proper ordering
+          LOG(WARNING) << "This series contains 2 slices with the same index, trying to display it anyway";
+          break;
+        }
       }
     }
 
@@ -342,8 +427,8 @@ namespace Orthanc
     if (!SortUsingPositions() &&
         !SortUsingIndexInSeries())
     {
-      LOG(ERROR) << "Unable to order the slices of the series " << seriesId;
-      throw OrthancException(ErrorCode_CannotOrderSlices);
+      throw OrthancException(ErrorCode_CannotOrderSlices,
+                             "Unable to order the slices of series " + seriesId);
     }
   }
 
@@ -361,28 +446,28 @@ namespace Orthanc
   }
 
 
-  const std::string& SliceOrdering::GetInstanceId(size_t index) const
+  const std::string& SliceOrdering::GetSortedInstanceId(size_t index) const
   {
-    if (index >= instances_.size())
+    if (index >= sortedInstances_.size())
     {
       throw OrthancException(ErrorCode_ParameterOutOfRange);
     }
     else
     {
-      return instances_[index]->GetIdentifier();
+      return sortedInstances_[index]->GetIdentifier();
     }
   }
 
 
-  unsigned int SliceOrdering::GetFramesCount(size_t index) const
+  unsigned int SliceOrdering::GetSortedInstanceFramesCount(size_t index) const
   {
-    if (index >= instances_.size())
+    if (index >= sortedInstances_.size())
     {
       throw OrthancException(ErrorCode_ParameterOutOfRange);
     }
     else
     {
-      return instances_[index]->GetFramesCount();
+      return sortedInstances_[index]->GetFramesCount();
     }
   }
 
@@ -393,9 +478,9 @@ namespace Orthanc
     result["Type"] = (isVolume_ ? "Volume" : "Sequence");
     
     Json::Value tmp = Json::arrayValue;
-    for (size_t i = 0; i < GetInstancesCount(); i++)
+    for (size_t i = 0; i < GetSortedInstancesCount(); i++)
     {
-      tmp.append(GetBasePath(ResourceType_Instance, GetInstanceId(i)) + "/file");
+      tmp.append(GetBasePath(ResourceType_Instance, GetSortedInstanceId(i)) + "/file");
     }
 
     result["Dicom"] = tmp;
@@ -403,18 +488,18 @@ namespace Orthanc
     Json::Value slicesShort = Json::arrayValue;
 
     tmp.clear();
-    for (size_t i = 0; i < GetInstancesCount(); i++)
+    for (size_t i = 0; i < GetSortedInstancesCount(); i++)
     {
-      std::string base = GetBasePath(ResourceType_Instance, GetInstanceId(i));
-      for (size_t j = 0; j < GetFramesCount(i); j++)
+      std::string base = GetBasePath(ResourceType_Instance, GetSortedInstanceId(i));
+      for (size_t j = 0; j < GetSortedInstanceFramesCount(i); j++)
       {
         tmp.append(base + "/frames/" + boost::lexical_cast<std::string>(j));
       }
 
       Json::Value tmp2 = Json::arrayValue;
-      tmp2.append(GetInstanceId(i));
+      tmp2.append(GetSortedInstanceId(i));
       tmp2.append(0);
-      tmp2.append(GetFramesCount(i));
+      tmp2.append(GetSortedInstanceFramesCount(i));
       
       slicesShort.append(tmp2);
     }
