@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,10 +33,12 @@
 
 #pragma once
 
-#include "ServerIndex.h"
+#include "../Core/DicomFormat/DicomMap.h"
 
 namespace Orthanc
 {
+  class ServerIndex;
+  
   class SliceOrdering
   {
   private:
@@ -49,9 +51,16 @@ namespace Orthanc
     std::string              seriesId_;
     bool                     hasNormal_;
     Vector                   normal_;
-    std::vector<Instance*>   instances_;
+    std::vector<Instance*>   instances_;        // this vector owns the instances
+    std::vector<Instance*>   sortedInstances_;  // this vectore references the instances of instances_
     bool                     isVolume_;
 
+    static bool ComputeNormal(Vector& normal,
+                              const DicomMap& dicom);
+
+    static bool IsParallelOrOpposite(const Vector& a,
+                                     const Vector& b);
+
     static bool IndexInSeriesComparator(const SliceOrdering::Instance* a,
                                         const SliceOrdering::Instance* b);
 
@@ -69,14 +78,14 @@ namespace Orthanc
 
     ~SliceOrdering();
 
-    size_t  GetInstancesCount() const
+    size_t  GetSortedInstancesCount() const
     {
-      return instances_.size();
+      return sortedInstances_.size();
     }
 
-    const std::string& GetInstanceId(size_t index) const;
+    const std::string& GetSortedInstanceId(size_t index) const;
 
-    unsigned int GetFramesCount(size_t index) const;
+    unsigned int GetSortedInstanceFramesCount(size_t index) const;
 
     void Format(Json::Value& result) const;
   };
