@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2018 Osimis S.A., Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,19 +36,21 @@
 
 #include <boost/algorithm/string/predicate.hpp>
 
-#include "../Core/Logging.h"
-#include "../Core/HttpServer/EmbeddedResourceHttpHandler.h"
-#include "../Core/HttpServer/FilesystemHttpHandler.h"
-#include "../Core/Lua/LuaFunctionCall.h"
 #include "../Core/DicomFormat/DicomArray.h"
 #include "../Core/DicomNetworking/DicomServer.h"
-#include "OrthancInitialization.h"
-#include "ServerContext.h"
-#include "OrthancFindRequestHandler.h"
-#include "OrthancMoveRequestHandler.h"
-#include "ServerToolbox.h"
-#include "../Plugins/Engine/OrthancPlugins.h"
 #include "../Core/DicomParsing/FromDcmtkBridge.h"
+#include "../Core/HttpServer/EmbeddedResourceHttpHandler.h"
+#include "../Core/HttpServer/FilesystemHttpHandler.h"
+#include "../Core/HttpServer/HttpServer.h"
+#include "../Core/Logging.h"
+#include "../Core/Lua/LuaFunctionCall.h"
+#include "../Plugins/Engine/OrthancPlugins.h"
+#include "OrthancConfiguration.h"
+#include "OrthancFindRequestHandler.h"
+#include "OrthancInitialization.h"
+#include "OrthancMoveRequestHandler.h"
+#include "ServerContext.h"
+#include "ServerToolbox.h"
 
 using namespace Orthanc;
 
@@ -89,19 +91,21 @@ public:
 
 
 
-class ModalitiesFromConfiguration : public Orthanc::DicomServer::IRemoteModalities
+class ModalitiesFromConfiguration : public DicomServer::IRemoteModalities
 {
 public:
   virtual bool IsSameAETitle(const std::string& aet1,
                              const std::string& aet2) 
   {
-    return Orthanc::Configuration::IsSameAETitle(aet1, aet2);
+    OrthancConfiguration::ReaderLock lock;
+    return lock.GetConfiguration().IsSameAETitle(aet1, aet2);
   }
 
   virtual bool LookupAETitle(RemoteModalityParameters& modality,
                              const std::string& aet) 
   {
-    return Orthanc::Configuration::LookupDicomModalityUsingAETitle(modality, aet);
+    OrthancConfiguration::ReaderLock lock;
+    return lock.GetConfiguration().LookupDicomModalityUsingAETitle(modality, aet);
   }
 };
 
@@ -128,8 +132,11 @@ public:
   {
     std::auto_ptr<OrthancFindRequestHandler> result(new OrthancFindRequestHandler(context_));
 
-    result->SetMaxResults(Configuration::GetGlobalUnsignedIntegerParameter("LimitFindResults", 0));
-    result->SetMaxInstances(Configuration::GetGlobalUnsignedIntegerParameter("LimitFindInstances", 0));
+    {
+      OrthancConfiguration::ReaderLock lock;
+      result->SetMaxResults(lock.GetConfiguration().GetUnsignedIntegerParameter("LimitFindResults", 0));
+      result->SetMaxInstances(lock.GetConfiguration().GetUnsignedIntegerParameter("LimitFindInstances", 0));
+    }
 
     if (result->GetMaxResults() == 0)
     {
@@ -176,8 +183,9 @@ public:
   OrthancApplicationEntityFilter(ServerContext& context) :
     context_(context)
   {
-    alwaysAllowEcho_ = Configuration::GetGlobalBoolParameter("DicomAlwaysAllowEcho", true);
-    alwaysAllowStore_ = Configuration::GetGlobalBoolParameter("DicomAlwaysAllowStore", true);
+    OrthancConfiguration::ReaderLock lock;
+    alwaysAllowEcho_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowEcho", true);
+    alwaysAllowStore_ = lock.GetConfiguration().GetBooleanParameter("DicomAlwaysAllowStore", true);
   }
 
   virtual bool IsAllowedConnection(const std::string& remoteIp,
@@ -187,9 +195,16 @@ public:
     LOG(INFO) << "Incoming connection from AET " << remoteAet
               << " on IP " << remoteIp << ", calling AET " << calledAet;
 
-    return (alwaysAllowEcho_ ||
-            alwaysAllowStore_ ||
-            Configuration::IsKnownAETitle(remoteAet, remoteIp));
+    if (alwaysAllowEcho_ ||
+        alwaysAllowStore_)
+    {
+      return true;
+    }
+    else
+    {
+      OrthancConfiguration::ReaderLock lock;
+      return lock.GetConfiguration().IsKnownAETitle(remoteAet, remoteIp);
+    }
   }
 
   virtual bool IsAllowedRequest(const std::string& remoteIp,
@@ -197,7 +212,7 @@ public:
                                 const std::string& calledAet,
                                 DicomRequestType type)
   {
-    LOG(INFO) << "Incoming " << Orthanc::EnumerationToString(type) << " request from AET "
+    LOG(INFO) << "Incoming " << EnumerationToString(type) << " request from AET "
               << remoteAet << " on IP " << remoteIp << ", calling AET " << calledAet;
     
     if (type == DicomRequestType_Echo &&
@@ -212,13 +227,19 @@ public:
       // Incoming C-Store requests are always accepted, even from unknown AET
       return true;
     }
-    else if (!Configuration::IsKnownAETitle(remoteAet, remoteIp))
-    {
-      return false;
-    }
     else
     {
-      return true;
+      OrthancConfiguration::ReaderLock lock;
+
+      RemoteModalityParameters modality;
+      if (lock.GetConfiguration().LookupDicomModalityUsingAETitle(modality, remoteAet))
+      {
+        return modality.IsRequestAllowed(type);
+      }
+      else
+      {
+        return false;
+      }
     }
   }
 
@@ -278,7 +299,10 @@ public:
       }
     }
 
-    return Configuration::GetGlobalBoolParameter(configuration, true);
+    {
+      OrthancConfiguration::ReaderLock lock;
+      return lock.GetConfiguration().GetBooleanParameter(configuration, true);
+    }
   }
 
 
@@ -303,7 +327,10 @@ public:
       }
     }
 
-    return Configuration::GetGlobalBoolParameter(configuration, false);
+    {
+      OrthancConfiguration::ReaderLock lock;
+      return lock.GetConfiguration().GetBooleanParameter(configuration, false);
+    }
   }
 };
 
@@ -329,11 +356,13 @@ public:
                          const IHttpHandler::Arguments& httpHeaders,
                          const IHttpHandler::GetArguments& getArguments)
   {
+#if ORTHANC_ENABLE_PLUGINS == 1
     if (plugins_ != NULL &&
         !plugins_->IsAllowed(method, uri, ip, username, httpHeaders, getArguments))
     {
       return false;
     }
+#endif
 
     static const char* HTTP_FILTER = "IncomingHttpRequestFilter";
 
@@ -455,6 +484,11 @@ public:
       message["OrthancError"] = EnumerationToString(errorCode);
       message["OrthancStatus"] = errorCode;
 
+      if (exception.HasDetails())
+      {
+        message["Details"] = exception.GetDetails();
+      }
+
       std::string info = message.toStyledString();
       output.SendStatus(httpStatus, info);
     }
@@ -482,6 +516,7 @@ static void PrintHelp(const char* path)
     << "  --logfile=[file]\tfile where to store the log of Orthanc" << std::endl
     << "\t\t\t(by default, the log is dumped to stderr)" << std::endl
     << "  --config=[file]\tcreate a sample configuration file and exit" << std::endl
+    << "\t\t\t(if file is \"-\", dumps to stdout)" << std::endl
     << "  --errors\t\tprint the supported error codes and exit" << std::endl
     << "  --verbose\t\tbe verbose in logs" << std::endl
     << "  --trace\t\thighest verbosity in logs (for debug)" << std::endl
@@ -508,12 +543,12 @@ static void PrintVersion(const char* path)
   std::cout
     << path << " " << ORTHANC_VERSION << std::endl
     << "Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics Department, University Hospital of Liege (Belgium)" << std::endl
-    << "Copyright (C) 2017-2018 Osimis S.A. (Belgium)" << std::endl
+    << "Copyright (C) 2017-2019 Osimis S.A. (Belgium)" << std::endl
     << "Licensing GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>, with OpenSSL exception." << std::endl
     << "This is free software: you are free to change and redistribute it." << std::endl
     << "There is NO WARRANTY, to the extent permitted by law." << std::endl
     << std::endl
-    << "Written by Sebastien Jodogne <s.jodogne@gmail.com>" << std::endl;
+    << "Written by Sebastien Jodogne <s.jodogne@orthanc-labs.com>" << std::endl;
 }
 
 
@@ -577,6 +612,7 @@ static void PrintErrors(const char* path)
     PrintErrorCode(ErrorCode_NullPointer, "Cannot handle a NULL pointer");
     PrintErrorCode(ErrorCode_DatabaseUnavailable, "The database is currently not available (probably a transient situation)");
     PrintErrorCode(ErrorCode_CanceledJob, "This job was canceled");
+    PrintErrorCode(ErrorCode_BadGeometry, "Geometry error encountered in Stone");
     PrintErrorCode(ErrorCode_SQLiteNotOpened, "SQLite: The database is not opened");
     PrintErrorCode(ErrorCode_SQLiteAlreadyOpened, "SQLite: Connection is already open");
     PrintErrorCode(ErrorCode_SQLiteCannotOpen, "SQLite: Unable to open the database");
@@ -636,6 +672,7 @@ static void PrintErrors(const char* path)
     PrintErrorCode(ErrorCode_CannotOrderSlices, "Unable to order the slices of the series");
     PrintErrorCode(ErrorCode_NoWorklistHandler, "No request handler factory for DICOM C-Find Modality SCP");
     PrintErrorCode(ErrorCode_AlreadyExistingTag, "Cannot override the value of a tag that already exists");
+    PrintErrorCode(ErrorCode_UnsupportedMediaType, "Unsupported media type");
   }
 
   std::cout << std::endl;
@@ -647,11 +684,22 @@ static void PrintErrors(const char* path)
 static void LoadPlugins(OrthancPlugins& plugins)
 {
   std::list<std::string> path;
-  Configuration::GetGlobalListOfStringsParameter(path, "Plugins");
+
+  {
+    OrthancConfiguration::ReaderLock lock;
+    lock.GetConfiguration().GetListOfStringsParameter(path, "Plugins");
+  }
+
   for (std::list<std::string>::const_iterator
          it = path.begin(); it != path.end(); ++it)
   {
-    std::string path = Configuration::InterpretStringParameterAsPath(*it);
+    std::string path;
+
+    {
+      OrthancConfiguration::ReaderLock lock;
+      path = lock.GetConfiguration().InterpretStringParameterAsPath(*it);
+    }
+
     LOG(WARNING) << "Loading plugin(s) from: " << path;
     plugins.GetManager().RegisterPlugin(path);
   }  
@@ -688,7 +736,8 @@ static bool WaitForExit(ServerContext& context,
     {
       // Handling of SIGHUP
 
-      if (Configuration::HasConfigurationChanged())
+      OrthancConfiguration::ReaderLock lock;
+      if (lock.GetConfiguration().HasConfigurationChanged())
       {
         LOG(WARNING) << "A SIGHUP signal has been received, resetting Orthanc";
         Logging::Flush();
@@ -697,7 +746,8 @@ static bool WaitForExit(ServerContext& context,
       }
       else
       {
-        LOG(WARNING) << "A SIGHUP signal has been received, but is ignored as the configuration has not changed";
+        LOG(WARNING) << "A SIGHUP signal has been received, but is ignored "
+                     << "as the configuration has not changed on the disk";
         Logging::Flush();
         continue;
       }
@@ -735,57 +785,156 @@ static bool StartHttpServer(ServerContext& context,
                             OrthancRestApi& restApi,
                             OrthancPlugins* plugins)
 {
-  if (!Configuration::GetGlobalBoolParameter("HttpServerEnabled", true))
+  bool httpServerEnabled;
+
+  {
+    OrthancConfiguration::ReaderLock lock;
+    httpServerEnabled = lock.GetConfiguration().GetBooleanParameter("HttpServerEnabled", true);
+  }
+
+  if (!httpServerEnabled)
   {
     LOG(WARNING) << "The HTTP server is disabled";
     return WaitForExit(context, restApi);
   }
-
-  MyHttpExceptionFormatter exceptionFormatter(Configuration::GetGlobalBoolParameter("HttpDescribeErrors", true), plugins);
-  
-
-  // HTTP server
-  MyIncomingHttpRequestFilter httpFilter(context, plugins);
-  MongooseServer httpServer;
-  httpServer.SetPortNumber(Configuration::GetGlobalUnsignedIntegerParameter("HttpPort", 8042));
-  httpServer.SetRemoteAccessAllowed(Configuration::GetGlobalBoolParameter("RemoteAccessAllowed", false));
-  httpServer.SetKeepAliveEnabled(Configuration::GetGlobalBoolParameter("KeepAlive", false));
-  httpServer.SetHttpCompressionEnabled(Configuration::GetGlobalBoolParameter("HttpCompressionEnabled", true));
-  httpServer.SetIncomingHttpRequestFilter(httpFilter);
-  httpServer.SetHttpExceptionFormatter(exceptionFormatter);
-
-  httpServer.SetAuthenticationEnabled(Configuration::GetGlobalBoolParameter("AuthenticationEnabled", false));
-  Configuration::SetupRegisteredUsers(httpServer);
-
-  if (Configuration::GetGlobalBoolParameter("SslEnabled", false))
-  {
-    std::string certificate = Configuration::InterpretStringParameterAsPath(
-      Configuration::GetGlobalStringParameter("SslCertificate", "certificate.pem"));
-    httpServer.SetSslEnabled(true);
-    httpServer.SetSslCertificate(certificate.c_str());
-  }
   else
   {
-    httpServer.SetSslEnabled(false);
-  }
+    MyIncomingHttpRequestFilter httpFilter(context, plugins);
+    HttpServer httpServer;
+    bool httpDescribeErrors;
 
-  httpServer.Register(context.GetHttpHandler());
-
-  if (httpServer.GetPortNumber() < 1024)
-  {
-    LOG(WARNING) << "The HTTP port is privileged (" 
-                 << httpServer.GetPortNumber() << " is below 1024), "
-                 << "make sure you run Orthanc as root/administrator";
-  }
-
-  httpServer.Start();
+#if ORTHANC_ENABLE_MONGOOSE == 1
+    const bool defaultKeepAlive = false;
+#elif ORTHANC_ENABLE_CIVETWEB == 1
+    const bool defaultKeepAlive = true;
+#else
+#  error "Either Mongoose or Civetweb must be enabled to compile this file"
+#endif
   
-  bool restart = WaitForExit(context, restApi);
+    {
+      OrthancConfiguration::ReaderLock lock;
+      
+      httpDescribeErrors = lock.GetConfiguration().GetBooleanParameter("HttpDescribeErrors", true);
+  
+      // HTTP server
+      httpServer.SetThreadsCount(lock.GetConfiguration().GetUnsignedIntegerParameter("HttpThreadsCount", 50));
+      httpServer.SetPortNumber(lock.GetConfiguration().GetUnsignedIntegerParameter("HttpPort", 8042));
+      httpServer.SetRemoteAccessAllowed(lock.GetConfiguration().GetBooleanParameter("RemoteAccessAllowed", false));
+      httpServer.SetKeepAliveEnabled(lock.GetConfiguration().GetBooleanParameter("KeepAlive", defaultKeepAlive));
+      httpServer.SetHttpCompressionEnabled(lock.GetConfiguration().GetBooleanParameter("HttpCompressionEnabled", true));
+      httpServer.SetTcpNoDelay(lock.GetConfiguration().GetBooleanParameter("TcpNoDelay", true));
+      httpServer.SetRequestTimeout(lock.GetConfiguration().GetUnsignedIntegerParameter("HttpRequestTimeout", 30));
 
-  httpServer.Stop();
-  LOG(WARNING) << "    HTTP server has stopped";
+      // Let's assume that the HTTP server is secure
+      context.SetHttpServerSecure(true);
 
-  return restart;
+      bool authenticationEnabled;
+      if (lock.GetConfiguration().LookupBooleanParameter(authenticationEnabled, "AuthenticationEnabled"))
+      {
+        httpServer.SetAuthenticationEnabled(authenticationEnabled);
+
+        if (httpServer.IsRemoteAccessAllowed() &&
+            !authenticationEnabled)
+        {
+          LOG(WARNING) << "====> Remote access is enabled while user authentication is explicitly disabled, "
+                       << "your setup is POSSIBLY INSECURE <====";
+          context.SetHttpServerSecure(false);
+        }
+      }
+      else if (httpServer.IsRemoteAccessAllowed())
+      {
+        // Starting with Orthanc 1.5.8, it is impossible to enable
+        // remote access without having explicitly disabled user
+        // authentication.
+        LOG(WARNING) << "Remote access is allowed but \"AuthenticationEnabled\" is not in the configuration, "
+                     << "automatically enabling HTTP authentication for security";          
+        httpServer.SetAuthenticationEnabled(true);
+      }
+      else
+      {
+        // If Orthanc only listens on the localhost, it is OK to have
+        // "AuthenticationEnabled" disabled
+        httpServer.SetAuthenticationEnabled(false);
+      }
+
+      bool hasUsers = lock.GetConfiguration().SetupRegisteredUsers(httpServer);
+
+      if (httpServer.IsAuthenticationEnabled() &&
+          !hasUsers)
+      {
+        if (httpServer.IsRemoteAccessAllowed())
+        {
+          /**
+           * Starting with Orthanc 1.5.8, if no user is explicitly
+           * defined while remote access is allowed, we create a
+           * default user, and Orthanc Explorer shows a warning
+           * message about an "Insecure setup". This convention is
+           * used in Docker images "jodogne/orthanc",
+           * "jodogne/orthanc-plugins" and "osimis/orthanc".
+           **/
+          LOG(WARNING) << "====> HTTP authentication is enabled, but no user is declared. "
+                       << "Creating a default user: Review your configuration option \"RegisteredUsers\". "
+                       << "Your setup is INSECURE <====";
+
+          context.SetHttpServerSecure(false);
+
+          // This is the username/password of the default user in Orthanc.
+          httpServer.RegisterUser("orthanc", "orthanc");
+        }
+        else
+        {
+          LOG(WARNING) << "HTTP authentication is enabled, but no user is declared, "
+                       << "check the value of configuration option \"RegisteredUsers\"";
+        }
+      }
+      
+      if (lock.GetConfiguration().GetBooleanParameter("SslEnabled", false))
+      {
+        std::string certificate = lock.GetConfiguration().InterpretStringParameterAsPath(
+          lock.GetConfiguration().GetStringParameter("SslCertificate", "certificate.pem"));
+        httpServer.SetSslEnabled(true);
+        httpServer.SetSslCertificate(certificate.c_str());
+      }
+      else
+      {
+        httpServer.SetSslEnabled(false);
+      }
+
+      if (lock.GetConfiguration().GetBooleanParameter("ExecuteLuaEnabled", false))
+      {
+        context.SetExecuteLuaEnabled(true);
+        LOG(WARNING) << "====> Remote LUA script execution is enabled.  Review your configuration option \"ExecuteLuaEnabled\". "
+                     << "Your setup is POSSIBLY INSECURE <====";
+      }
+      else
+      {
+        context.SetExecuteLuaEnabled(false);
+        LOG(WARNING) << "Remote LUA script execution is disabled";
+      }
+    }
+
+    MyHttpExceptionFormatter exceptionFormatter(httpDescribeErrors, plugins);
+        
+    httpServer.SetIncomingHttpRequestFilter(httpFilter);
+    httpServer.SetHttpExceptionFormatter(exceptionFormatter);
+    httpServer.Register(context.GetHttpHandler());
+
+    if (httpServer.GetPortNumber() < 1024)
+    {
+      LOG(WARNING) << "The HTTP port is privileged (" 
+                   << httpServer.GetPortNumber() << " is below 1024), "
+                   << "make sure you run Orthanc as root/administrator";
+    }
+
+    httpServer.Start();
+  
+    bool restart = WaitForExit(context, restApi);
+
+    httpServer.Stop();
+    LOG(WARNING) << "    HTTP server has stopped";
+
+    return restart;
+  }
 }
 
 
@@ -793,89 +942,102 @@ static bool StartDicomServer(ServerContext& context,
                              OrthancRestApi& restApi,
                              OrthancPlugins* plugins)
 {
-  if (!Configuration::GetGlobalBoolParameter("DicomServerEnabled", true))
+  bool dicomServerEnabled;
+
+  {
+    OrthancConfiguration::ReaderLock lock;
+    dicomServerEnabled = lock.GetConfiguration().GetBooleanParameter("DicomServerEnabled", true);
+  }
+
+  if (!dicomServerEnabled)
   {
     LOG(WARNING) << "The DICOM server is disabled";
     return StartHttpServer(context, restApi, plugins);
   }
-
-  MyDicomServerFactory serverFactory(context);
-  OrthancApplicationEntityFilter dicomFilter(context);
-  ModalitiesFromConfiguration modalities;
+  else
+  {
+    MyDicomServerFactory serverFactory(context);
+    OrthancApplicationEntityFilter dicomFilter(context);
+    ModalitiesFromConfiguration modalities;
   
-  // Setup the DICOM server  
-  DicomServer dicomServer;
-  dicomServer.SetRemoteModalities(modalities);
-  dicomServer.SetCalledApplicationEntityTitleCheck(Configuration::GetGlobalBoolParameter("DicomCheckCalledAet", false));
-  dicomServer.SetStoreRequestHandlerFactory(serverFactory);
-  dicomServer.SetMoveRequestHandlerFactory(serverFactory);
-  dicomServer.SetFindRequestHandlerFactory(serverFactory);
-  dicomServer.SetAssociationTimeout(Configuration::GetGlobalUnsignedIntegerParameter("DicomScpTimeout", 30));
+    // Setup the DICOM server  
+    DicomServer dicomServer;
+    dicomServer.SetRemoteModalities(modalities);
+    dicomServer.SetStoreRequestHandlerFactory(serverFactory);
+    dicomServer.SetMoveRequestHandlerFactory(serverFactory);
+    dicomServer.SetFindRequestHandlerFactory(serverFactory);
 
+    {
+      OrthancConfiguration::ReaderLock lock;
+      dicomServer.SetCalledApplicationEntityTitleCheck(lock.GetConfiguration().GetBooleanParameter("DicomCheckCalledAet", false));
+      dicomServer.SetAssociationTimeout(lock.GetConfiguration().GetUnsignedIntegerParameter("DicomScpTimeout", 30));
+      dicomServer.SetPortNumber(lock.GetConfiguration().GetUnsignedIntegerParameter("DicomPort", 4242));
+      dicomServer.SetApplicationEntityTitle(lock.GetConfiguration().GetStringParameter("DicomAet", "ORTHANC"));
+    }
 
 #if ORTHANC_ENABLE_PLUGINS == 1
-  if (plugins != NULL)
-  {
-    if (plugins->HasWorklistHandler())
+    if (plugins != NULL)
     {
-      dicomServer.SetWorklistRequestHandlerFactory(*plugins);
-    }
+      if (plugins->HasWorklistHandler())
+      {
+        dicomServer.SetWorklistRequestHandlerFactory(*plugins);
+      }
 
-    if (plugins->HasFindHandler())
-    {
-      dicomServer.SetFindRequestHandlerFactory(*plugins);
-    }
+      if (plugins->HasFindHandler())
+      {
+        dicomServer.SetFindRequestHandlerFactory(*plugins);
+      }
 
-    if (plugins->HasMoveHandler())
-    {
-      dicomServer.SetMoveRequestHandlerFactory(*plugins);
+      if (plugins->HasMoveHandler())
+      {
+        dicomServer.SetMoveRequestHandlerFactory(*plugins);
+      }
     }
-  }
 #endif
 
-  dicomServer.SetPortNumber(Configuration::GetGlobalUnsignedIntegerParameter("DicomPort", 4242));
-  dicomServer.SetApplicationEntityTitle(Configuration::GetGlobalStringParameter("DicomAet", "ORTHANC"));
-  dicomServer.SetApplicationEntityFilter(dicomFilter);
+    dicomServer.SetApplicationEntityFilter(dicomFilter);
 
-  if (dicomServer.GetPortNumber() < 1024)
-  {
-    LOG(WARNING) << "The DICOM port is privileged (" 
-                 << dicomServer.GetPortNumber() << " is below 1024), "
-                 << "make sure you run Orthanc as root/administrator";
+    if (dicomServer.GetPortNumber() < 1024)
+    {
+      LOG(WARNING) << "The DICOM port is privileged (" 
+                   << dicomServer.GetPortNumber() << " is below 1024), "
+                   << "make sure you run Orthanc as root/administrator";
+    }
+
+    dicomServer.Start();
+    LOG(WARNING) << "DICOM server listening with AET " << dicomServer.GetApplicationEntityTitle() 
+                 << " on port: " << dicomServer.GetPortNumber();
+
+    bool restart = false;
+    ErrorCode error = ErrorCode_Success;
+
+    try
+    {
+      restart = StartHttpServer(context, restApi, plugins);
+    }
+    catch (OrthancException& e)
+    {
+      error = e.GetErrorCode();
+    }
+
+    dicomServer.Stop();
+    LOG(WARNING) << "    DICOM server has stopped";
+
+    serverFactory.Done();
+
+    if (error != ErrorCode_Success)
+    {
+      throw OrthancException(error);
+    }
+
+    return restart;
   }
-
-  dicomServer.Start();
-  LOG(WARNING) << "DICOM server listening with AET " << dicomServer.GetApplicationEntityTitle() 
-               << " on port: " << dicomServer.GetPortNumber();
-
-  bool restart = false;
-  ErrorCode error = ErrorCode_Success;
-
-  try
-  {
-    restart = StartHttpServer(context, restApi, plugins);
-  }
-  catch (OrthancException& e)
-  {
-    error = e.GetErrorCode();
-  }
-
-  dicomServer.Stop();
-  LOG(WARNING) << "    DICOM server has stopped";
-
-  serverFactory.Done();
-
-  if (error != ErrorCode_Success)
-  {
-    throw OrthancException(error);
-  }
-
-  return restart;
 }
 
 
 static bool ConfigureHttpHandler(ServerContext& context,
-                                 OrthancPlugins *plugins)
+                                 OrthancPlugins *plugins,
+                                 bool loadJobsFromDatabase)
 {
 #if ORTHANC_ENABLE_PLUGINS == 1
   // By order of priority, first apply the "plugins" layer, so that
@@ -900,7 +1062,13 @@ static bool ConfigureHttpHandler(ServerContext& context,
   OrthancRestApi restApi(context);
   context.GetHttpHandler().Register(restApi, true);
 
-  return StartDicomServer(context, restApi, plugins);
+  context.SetupJobsEngine(false /* not running unit tests */, loadJobsFromDatabase);
+
+  bool restart = StartDicomServer(context, restApi, plugins);
+
+  context.Stop();
+
+  return restart;
 }
 
 
@@ -922,9 +1090,10 @@ static void UpgradeDatabase(IDatabaseWrapper& database,
 
   if (currentVersion > ORTHANC_DATABASE_VERSION)
   {
-    LOG(ERROR) << "The version of the database schema (" << currentVersion
-               << ") is too recent for this version of Orthanc. Please upgrade Orthanc.";
-    throw OrthancException(ErrorCode_IncompatibleDatabaseVersion);
+    throw OrthancException(ErrorCode_IncompatibleDatabaseVersion,
+                           "The version of the database schema (" +
+                           boost::lexical_cast<std::string>(currentVersion) +
+                           ") is too recent for this version of Orthanc. Please upgrade Orthanc.");
   }
 
   LOG(WARNING) << "Upgrading the database from schema version "
@@ -937,7 +1106,7 @@ static void UpgradeDatabase(IDatabaseWrapper& database,
   catch (OrthancException&)
   {
     LOG(ERROR) << "Unable to run the automated upgrade, please use the replication instructions: "
-               << "http://book.orthanc-server.com//users/replication.html";
+               << "http://book.orthanc-server.com/users/replication.html";
     throw;
   }
     
@@ -945,8 +1114,9 @@ static void UpgradeDatabase(IDatabaseWrapper& database,
   currentVersion = database.GetDatabaseVersion();
   if (ORTHANC_DATABASE_VERSION != currentVersion)
   {
-    LOG(ERROR) << "The database schema was not properly upgraded, it is still at version " << currentVersion;
-    throw OrthancException(ErrorCode_IncompatibleDatabaseVersion);
+    throw OrthancException(ErrorCode_IncompatibleDatabaseVersion,
+                           "The database schema was not properly upgraded, it is still at version " +
+                           boost::lexical_cast<std::string>(currentVersion));
   }
   else
   {
@@ -956,84 +1126,129 @@ static void UpgradeDatabase(IDatabaseWrapper& database,
 }
 
 
+
+namespace
+{
+  class ServerContextConfigurator : public boost::noncopyable
+  {
+  private:
+    ServerContext&   context_;
+    OrthancPlugins*  plugins_;
+
+  public:
+    ServerContextConfigurator(ServerContext& context,
+                              OrthancPlugins* plugins) :
+      context_(context),
+      plugins_(plugins)
+    {
+      {
+        OrthancConfiguration::WriterLock lock;
+        lock.GetConfiguration().SetServerIndex(context.GetIndex());
+      }
+
+#if ORTHANC_ENABLE_PLUGINS == 1
+      if (plugins_ != NULL)
+      {
+        plugins_->SetServerContext(context_);
+        context_.SetPlugins(*plugins_);
+      }
+#endif
+    }
+
+    ~ServerContextConfigurator()
+    {
+      {
+        OrthancConfiguration::WriterLock lock;
+        lock.GetConfiguration().ResetServerIndex();
+      }
+
+#if ORTHANC_ENABLE_PLUGINS == 1
+      if (plugins_ != NULL)
+      {
+        plugins_->ResetServerContext();
+        context_.ResetPlugins();
+      }
+#endif
+    }
+  };
+}
+
+
 static bool ConfigureServerContext(IDatabaseWrapper& database,
                                    IStorageArea& storageArea,
                                    OrthancPlugins *plugins,
                                    bool loadJobsFromDatabase)
 {
-  // These configuration options must be set before creating the
-  // ServerContext, otherwise the possible Lua scripts will not be
-  // able to properly issue HTTP/HTTPS queries
-  HttpClient::ConfigureSsl(Configuration::GetGlobalBoolParameter("HttpsVerifyPeers", true),
-                           Configuration::InterpretStringParameterAsPath
-                           (Configuration::GetGlobalStringParameter("HttpsCACertificates", "")));
-  HttpClient::SetDefaultTimeout(Configuration::GetGlobalUnsignedIntegerParameter("HttpTimeout", 0));
-  HttpClient::SetDefaultProxy(Configuration::GetGlobalStringParameter("HttpProxy", ""));
-
-  DicomUserConnection::SetDefaultTimeout(Configuration::GetGlobalUnsignedIntegerParameter("DicomScuTimeout", 10));
-
-  ServerContext context(database, storageArea, false /* not running unit tests */, loadJobsFromDatabase);
-  context.SetCompressionEnabled(Configuration::GetGlobalBoolParameter("StorageCompression", false));
-  context.SetStoreMD5ForAttachments(Configuration::GetGlobalBoolParameter("StoreMD5ForAttachments", true));
-
-  try
+  size_t maxCompletedJobs;
+  
   {
-    context.GetIndex().SetMaximumPatientCount(Configuration::GetGlobalUnsignedIntegerParameter("MaximumPatientCount", 0));
+    OrthancConfiguration::ReaderLock lock;
+
+    // These configuration options must be set before creating the
+    // ServerContext, otherwise the possible Lua scripts will not be
+    // able to properly issue HTTP/HTTPS queries
+    HttpClient::ConfigureSsl(lock.GetConfiguration().GetBooleanParameter("HttpsVerifyPeers", true),
+                             lock.GetConfiguration().InterpretStringParameterAsPath
+                             (lock.GetConfiguration().GetStringParameter("HttpsCACertificates", "")));
+    HttpClient::SetDefaultVerbose(lock.GetConfiguration().GetBooleanParameter("HttpVerbose", false));
+
+    // The value "0" below makes the class HttpClient use its default
+    // value (DEFAULT_HTTP_TIMEOUT = 60 seconds in Orthanc 1.5.7)
+    HttpClient::SetDefaultTimeout(lock.GetConfiguration().GetUnsignedIntegerParameter("HttpTimeout", 0));
+    
+    HttpClient::SetDefaultProxy(lock.GetConfiguration().GetStringParameter("HttpProxy", ""));
+    
+    DicomUserConnection::SetDefaultTimeout(lock.GetConfiguration().GetUnsignedIntegerParameter("DicomScuTimeout", 10));
+
+    maxCompletedJobs = lock.GetConfiguration().GetUnsignedIntegerParameter("JobsHistorySize", 10);
+
+    if (maxCompletedJobs == 0)
+    {
+      LOG(WARNING) << "Setting option \"JobsHistorySize\" to zero is not recommended";
+    }
   }
-  catch (...)
+  
+  ServerContext context(database, storageArea, false /* not running unit tests */, maxCompletedJobs);
+
   {
-    context.GetIndex().SetMaximumPatientCount(0);
+    OrthancConfiguration::ReaderLock lock;
+
+    context.SetCompressionEnabled(lock.GetConfiguration().GetBooleanParameter("StorageCompression", false));
+    context.SetStoreMD5ForAttachments(lock.GetConfiguration().GetBooleanParameter("StoreMD5ForAttachments", true));
+
+    // New option in Orthanc 1.4.2
+    context.GetIndex().SetOverwriteInstances(lock.GetConfiguration().GetBooleanParameter("OverwriteInstances", false));
+
+    try
+    {
+      context.GetIndex().SetMaximumPatientCount(lock.GetConfiguration().GetUnsignedIntegerParameter("MaximumPatientCount", 0));
+    }
+    catch (...)
+    {
+      context.GetIndex().SetMaximumPatientCount(0);
+    }
+
+    try
+    {
+      uint64_t size = lock.GetConfiguration().GetUnsignedIntegerParameter("MaximumStorageSize", 0);
+      context.GetIndex().SetMaximumStorageSize(size * 1024 * 1024);
+    }
+    catch (...)
+    {
+      context.GetIndex().SetMaximumStorageSize(0);
+    }
   }
 
-  try
   {
-    uint64_t size = Configuration::GetGlobalUnsignedIntegerParameter("MaximumStorageSize", 0);
-    context.GetIndex().SetMaximumStorageSize(size * 1024 * 1024);
+    ServerContextConfigurator configurator(context, plugins);
+
+    {
+      OrthancConfiguration::WriterLock lock;
+      lock.GetConfiguration().LoadModalitiesAndPeers();
+    }
+
+    return ConfigureHttpHandler(context, plugins, loadJobsFromDatabase);
   }
-  catch (...)
-  {
-    context.GetIndex().SetMaximumStorageSize(0);
-  }
-
-  context.GetJobsEngine().GetRegistry().SetMaxCompletedJobs
-    (Configuration::GetGlobalUnsignedIntegerParameter("JobsHistorySize", 10));
-
-#if ORTHANC_ENABLE_PLUGINS == 1
-  if (plugins)
-  {
-    plugins->SetServerContext(context);
-    context.SetPlugins(*plugins);
-  }
-#endif
-
-  bool restart = false;
-  ErrorCode error = ErrorCode_Success;
-
-  try
-  {
-    restart = ConfigureHttpHandler(context, plugins);
-  }
-  catch (OrthancException& e)
-  {
-    error = e.GetErrorCode();
-  }
-
-  context.Stop();
-
-#if ORTHANC_ENABLE_PLUGINS == 1
-  if (plugins)
-  {
-    plugins->ResetServerContext();
-    context.ResetPlugins();
-  }
-#endif
-
-  if (error != ErrorCode_Success)
-  {
-    throw OrthancException(error);
-  }
-
-  return restart;
 }
 
 
@@ -1054,10 +1269,11 @@ static bool ConfigureDatabase(IDatabaseWrapper& database,
   }
   else if (currentVersion != ORTHANC_DATABASE_VERSION)
   {
-    LOG(ERROR) << "The database schema must be changed from version "
-               << currentVersion << " to " << ORTHANC_DATABASE_VERSION 
-               << ": Please run Orthanc with the \"--upgrade\" argument";
-    throw OrthancException(ErrorCode_IncompatibleDatabaseVersion);
+    throw OrthancException(ErrorCode_IncompatibleDatabaseVersion,
+                           "The database schema must be upgraded from version " +
+                           boost::lexical_cast<std::string>(currentVersion) + " to " +
+                           boost::lexical_cast<std::string>(ORTHANC_DATABASE_VERSION) +
+                           ": Please run Orthanc with the \"--upgrade\" argument");
   }
 
   bool success = ConfigureServerContext
@@ -1090,7 +1306,7 @@ static bool ConfigurePlugins(int argc,
   }
   else
   {
-    databasePtr.reset(Configuration::CreateDatabaseWrapper());
+    databasePtr.reset(CreateDatabaseWrapper());
     database = databasePtr.get();
   }
 
@@ -1101,7 +1317,7 @@ static bool ConfigurePlugins(int argc,
   }
   else
   {
-    storage.reset(Configuration::CreateStorageArea());
+    storage.reset(CreateStorageArea());
   }
 
   assert(database != NULL);
@@ -1112,8 +1328,12 @@ static bool ConfigurePlugins(int argc,
 
 #elif ORTHANC_ENABLE_PLUGINS == 0
   // The plugins are disabled
-  databasePtr.reset(Configuration::CreateDatabaseWrapper());
-  storage.reset(Configuration::CreateStorageArea());
+
+  databasePtr.reset(CreateDatabaseWrapper());
+  storage.reset(CreateStorageArea());
+
+  assert(databasePtr.get() != NULL);
+  assert(storage.get() != NULL);
 
   return ConfigureDatabase(*databasePtr, *storage, NULL,
                            upgradeDatabase, loadJobsFromDatabase);
@@ -1256,7 +1476,15 @@ int main(int argc, char* argv[])
 
       try
       {
-        SystemToolbox::WriteFile(configurationSample, target);
+        if (target == "-")
+        {
+          // New in 1.5.8: Print to stdout
+          std::cout << configurationSample;
+        }
+        else
+        {
+          SystemToolbox::WriteFile(configurationSample, target);
+        }
         return 0;
       }
       catch (OrthancException&)
