@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -59,6 +59,7 @@ namespace Orthanc
 #include "../../Core/FileStorage/IStorageArea.h"
 #include "../../Core/HttpServer/IHttpHandler.h"
 #include "../../Core/HttpServer/IIncomingHttpRequestFilter.h"
+#include "../../Core/JobsEngine/IJob.h"
 #include "../../OrthancServer/IDicomImageDecoder.h"
 #include "../../OrthancServer/IServerListener.h"
 #include "OrthancPluginDatabase.h"
@@ -88,10 +89,21 @@ namespace Orthanc
     class WorklistHandler;
     class FindHandler;
     class MoveHandler;
-
+    class HttpClientChunkedRequest;
+    class HttpClientChunkedAnswer;
+    class HttpServerChunkedReader;
+    
     void RegisterRestCallback(const void* parameters,
                               bool lock);
 
+    void RegisterChunkedRestCallback(const void* parameters);
+
+    bool HandleChunkedGetDelete(HttpOutput& output,
+                                HttpMethod method,
+                                const UriComponents& uri,
+                                const Arguments& headers,
+                                const GetArguments& getArguments);
+
     void RegisterOnStoredInstanceCallback(const void* parameters);
 
     void RegisterOnChangeCallback(const void* parameters);
@@ -104,10 +116,14 @@ namespace Orthanc
 
     void RegisterDecodeImageCallback(const void* parameters);
 
+    void RegisterJobsUnserializer(const void* parameters);
+
     void RegisterIncomingHttpRequestFilter(const void* parameters);
 
     void RegisterIncomingHttpRequestFilter2(const void* parameters);
 
+    void RegisterRefreshMetricsCallback(const void* parameters);
+
     void AnswerBuffer(const void* parameters);
 
     void Redirect(const void* parameters);
@@ -145,6 +161,8 @@ namespace Orthanc
 
     void SetHttpHeader(const void* parameters);
 
+    void SetHttpErrorDetails(const void* parameters);
+
     void BufferCompression(const void* parameters);
 
     void UncompressImage(const void* parameters);
@@ -157,6 +175,10 @@ namespace Orthanc
 
     void CallHttpClient2(const void* parameters);
 
+    void ChunkedHttpClient(const void* parameters);
+
+    void CallPeerApi(const void* parameters);
+  
     void GetFontInfo(const void* parameters);
 
     void DrawText(const void* parameters);
@@ -181,6 +203,8 @@ namespace Orthanc
     void ComputeHash(_OrthancPluginService service,
                      const void* parameters);
 
+    void GetTagName(const void* parameters);
+
     void SignalChangeInternal(OrthancPluginChangeType changeType,
                               OrthancPluginResourceType resourceType,
                               const char* resource);
@@ -210,7 +234,7 @@ namespace Orthanc
                         const UriComponents& uri,
                         const Arguments& headers,
                         const GetArguments& getArguments,
-                        const char* bodyData,
+                        const void* bodyData,
                         size_t bodySize);
 
     virtual bool InvokeService(SharedLibrary& plugin,
@@ -262,6 +286,16 @@ namespace Orthanc
       SignalChangeInternal(OrthancPluginChangeType_OrthancStopped, OrthancPluginResourceType_None, NULL);
     }
 
+    void SignalUpdatedPeers()
+    {
+      SignalChangeInternal(OrthancPluginChangeType_UpdatedPeers, OrthancPluginResourceType_None, NULL);
+    }
+
+    void SignalUpdatedModalities()
+    {
+      SignalChangeInternal(OrthancPluginChangeType_UpdatedModalities, OrthancPluginResourceType_None, NULL);
+    }
+
     bool HasWorklistHandler();
 
     virtual IWorklistRequestHandler* ConstructWorklistRequestHandler();
@@ -284,7 +318,7 @@ namespace Orthanc
                            const char* ip,
                            const char* username,
                            const IHttpHandler::Arguments& httpHeaders,
-                           const IHttpHandler::GetArguments& getArguments) const;
+                           const IHttpHandler::GetArguments& getArguments);
 
     bool HasFindHandler();
 
@@ -293,6 +327,20 @@ namespace Orthanc
     bool HasMoveHandler();
 
     virtual IMoveRequestHandler* ConstructMoveRequestHandler();
+
+    IJob* UnserializeJob(const std::string& type,
+                         const Json::Value& value);
+
+    void RefreshMetrics();
+
+    // New in Orthanc 1.5.7
+    virtual bool CreateChunkedRequestReader(std::auto_ptr<IChunkedRequestReader>& target,
+                                            RequestOrigin origin,
+                                            const char* remoteIp,
+                                            const char* username,
+                                            HttpMethod method,
+                                            const UriComponents& uri,
+                                            const Arguments& headers);
   };
 }
 
