@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -50,6 +50,8 @@ namespace OrthancPlugins
 
     const Prefix& GetPrefixItem(size_t depth) const;
 
+    Prefix& GetPrefixItem(size_t depth);
+
   public:
     DicomPath(const DicomTag& finalTag) :
     finalTag_(finalTag)
@@ -94,6 +96,12 @@ namespace OrthancPlugins
     {
       return GetPrefixItem(depth).second;
     }
+
+    void SetPrefixIndex(size_t depth,
+                        size_t value)
+    {
+      GetPrefixItem(depth).second = value;
+    }
     
     const DicomTag& GetFinalTag() const
     {
