@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -54,7 +54,7 @@ static OrthancPluginErrorCode DecodeImageCallback(OrthancPluginImage** target,
     *target = NULL;
 
     std::string s = "Cannot decode image using GDCM: " + std::string(e.what());
-    OrthancPluginLogError(context_, s.c_str());
+    OrthancPluginLogInfo(context_, s.c_str());
     return OrthancPluginErrorCode_Plugin;
   }
 }
