@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017 Osimis, Belgium
+ * Copyright (C) 2017-2019 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,6 +35,7 @@
 #include "gtest/gtest.h"
 
 #include <memory>
+#include <algorithm>
 #include <boost/thread.hpp>
 #include <boost/lexical_cast.hpp>
 
@@ -170,6 +171,24 @@ TEST(LRU, PayloadUpdateBis)
   ASSERT_TRUE(r.IsEmpty());
 }
 
+TEST(LRU, GetAllKeys)
+{
+  Orthanc::LeastRecentlyUsedIndex<std::string, int> r;
+  std::vector<std::string> keys;
+
+  r.AddOrMakeMostRecent("a", 420);
+  r.GetAllKeys(keys);
+
+  ASSERT_EQ(1u, keys.size());
+  ASSERT_EQ("a", keys[0]);
+
+  r.AddOrMakeMostRecent("b", 421);
+  r.GetAllKeys(keys);
+
+  ASSERT_EQ(2u, keys.size());
+  ASSERT_TRUE(std::find(keys.begin(), keys.end(),"a") != keys.end());
+  ASSERT_TRUE(std::find(keys.begin(), keys.end(),"b") != keys.end());
+}
 
 
 
@@ -186,7 +205,7 @@ namespace
     {
     }
 
-    virtual ~Integer()
+    virtual ~Integer() ORTHANC_OVERRIDE
     {
       LOG(INFO) << "Removing cache entry for " << value_;
       log_ += boost::lexical_cast<std::string>(value_) + " ";
@@ -198,7 +217,7 @@ namespace
   public:
     std::string log_;
 
-    Orthanc::IDynamicObject* Provide(const std::string& s)
+    virtual Orthanc::IDynamicObject* Provide(const std::string& s) ORTHANC_OVERRIDE
     {
       LOG(INFO) << "Providing " << s;
       return new Integer(log_, boost::lexical_cast<int>(s));
@@ -261,9 +280,25 @@ TEST(LRU, SharedArchive)
   for (int i = 1; i < 100; i++)
   {
     a.Add(new S("Item " + boost::lexical_cast<std::string>(i)));
+    
     // Continuously protect the two first items
-    try { Orthanc::SharedArchive::Accessor(a, first);  } catch (Orthanc::OrthancException&) {}
-    try { Orthanc::SharedArchive::Accessor(a, second); } catch (Orthanc::OrthancException&) {}
+    {
+      Orthanc::SharedArchive::Accessor accessor(a, first);
+      ASSERT_TRUE(accessor.IsValid());
+      ASSERT_EQ("First item", dynamic_cast<S&>(accessor.GetItem()).GetValue());
+    }
+
+    {
+      Orthanc::SharedArchive::Accessor accessor(a, second);
+      ASSERT_TRUE(accessor.IsValid());
+      ASSERT_EQ("Second item", dynamic_cast<S&>(accessor.GetItem()).GetValue());
+    }
+
+    {
+      Orthanc::SharedArchive::Accessor accessor(a, "nope");
+      ASSERT_FALSE(accessor.IsValid());
+      ASSERT_THROW(accessor.GetItem(), Orthanc::OrthancException);
+    }
   }
 
   std::list<std::string> i;
